@@ -78,7 +78,7 @@ typedef struct {
 typedef struct {
 	slurmdb_cluster_rec_t *cluster;
 	job_desc_msg_t        *req;
-	List                   resp_msg_list;
+	list_t *resp_msg_list;
 } load_willrun_req_struct_t;
 
 typedef struct {
@@ -183,9 +183,7 @@ slurm_allocate_resources_blocking (const job_desc_msg_t *user_req,
 
 	/* make a copy of the user's job description struct so that we
 	 * can make changes before contacting the controller */
-	req = (job_desc_msg_t *)xmalloc(sizeof(job_desc_msg_t));
-	if (req == NULL)
-		return NULL;
+	req = xmalloc(sizeof(job_desc_msg_t));
 	memcpy(req, user_req, sizeof(job_desc_msg_t));
 
 	/*
@@ -295,22 +293,22 @@ static void *_load_willrun_thread(void *args)
 	list_append(load_args->resp_msg_list, resp);
 	xfree(args);
 
-	return (void *) NULL;
+	return NULL;
 }
 
 static int _fed_job_will_run(job_desc_msg_t *req,
 			     will_run_response_msg_t **will_run_resp,
 			     slurmdb_federation_rec_t *fed)
 {
-	List resp_msg_list;
+	list_t *resp_msg_list;
 	int pthread_count = 0, i;
 	pthread_t *load_thread = 0;
 	load_willrun_req_struct_t *load_args;
-	ListIterator iter;
+	list_itr_t *iter;
 	will_run_response_msg_t *earliest_resp = NULL;
 	load_willrun_resp_struct_t *tmp_resp;
 	slurmdb_cluster_rec_t *cluster;
-	List req_clusters = NULL;
+	list_t *req_clusters = NULL;
 
 	xassert(req);
 	xassert(will_run_resp);
@@ -328,8 +326,7 @@ static int _fed_job_will_run(job_desc_msg_t *req,
 
 	/* Spawn one pthread per cluster to collect job information */
 	resp_msg_list = list_create(NULL);
-	load_thread = xmalloc(sizeof(pthread_t) *
-			      list_count(fed->cluster_list));
+	load_thread = xcalloc(list_count(fed->cluster_list), sizeof(pthread_t));
 	iter = list_iterator_create(fed->cluster_list);
 	while ((cluster = (slurmdb_cluster_rec_t *)list_next(iter))) {
 		if ((cluster->control_host == NULL) ||
@@ -354,13 +351,13 @@ static int _fed_job_will_run(job_desc_msg_t *req,
 
 	/* Wait for all pthreads to complete */
 	for (i = 0; i < pthread_count; i++)
-		pthread_join(load_thread[i], NULL);
+		slurm_thread_join(load_thread[i]);
 	xfree(load_thread);
 
 	iter = list_iterator_create(resp_msg_list);
 	while ((tmp_resp = (load_willrun_resp_struct_t *)list_next(iter))) {
 		if (!tmp_resp->willrun_resp_msg)
-			slurm_seterrno(tmp_resp->rc);
+			errno = tmp_resp->rc;
 		else if ((!earliest_resp) ||
 			 (tmp_resp->willrun_resp_msg->start_time <
 			  earliest_resp->start_time)) {
@@ -384,11 +381,12 @@ static int _fed_job_will_run(job_desc_msg_t *req,
 }
 
 /* Get total node count and lead job ID from RESPONSE_HET_JOB_ALLOCATION */
-static void _het_job_alloc_test(List resp, uint32_t *node_cnt, uint32_t *job_id)
+static void _het_job_alloc_test(list_t *resp, uint32_t *node_cnt,
+				uint32_t *job_id)
 {
 	resource_allocation_response_msg_t *alloc;
 	uint32_t inx = 0, het_job_node_cnt = 0, het_job_id = 0;
-	ListIterator iter;
+	list_itr_t *iter;
 
 	xassert(resp);
 	iter = list_iterator_create(resp);
@@ -425,17 +423,18 @@ static void _het_job_alloc_test(List resp, uint32_t *node_cnt, uint32_t *job_id)
  *      with no allocation granted)
  * NOTE: free the response using list_destroy()
  */
-List slurm_allocate_het_job_blocking(List job_req_list, time_t timeout,
-				     void(*pending_callback)(uint32_t job_id))
+list_t *slurm_allocate_het_job_blocking(
+	list_t *job_req_list, time_t timeout,
+	void (*pending_callback) (uint32_t job_id))
 {
 	int rc;
 	slurm_msg_t req_msg;
 	slurm_msg_t resp_msg;
-	List resp = NULL;
+	list_t *resp = NULL;
 	job_desc_msg_t *req;
 	listen_t *listen = NULL;
 	int errnum = SLURM_SUCCESS;
-	ListIterator iter;
+	list_itr_t *iter;
 	bool immediate_flag = false;
 	uint32_t node_cnt = 0, job_id = 0;
 	bool already_done = false;
@@ -494,7 +493,7 @@ List slurm_allocate_het_job_blocking(List job_req_list, time_t timeout,
 	case RESPONSE_HET_JOB_ALLOCATION:
 		/* Yay, the controller has acknowledged our request!
 		 * Test if we have an allocation yet? */
-		resp = (List) resp_msg.data;
+		resp = resp_msg.data;
 		_het_job_alloc_test(resp, &node_cnt, &job_id);
 		if (node_cnt > 0) {
 			/* yes, allocation has been granted */
@@ -572,7 +571,7 @@ int slurm_job_will_run(job_desc_msg_t *req)
 		     will_run_resp->node_list,
 		     will_run_resp->part_name);
 		if (will_run_resp->preemptee_job_id) {
-			ListIterator itr;
+			list_itr_t *itr;
 			uint32_t *job_id_ptr;
 			char *job_list = NULL, *sep = "";
 			itr = list_iterator_create(will_run_resp->
@@ -603,16 +602,16 @@ int slurm_job_will_run(job_desc_msg_t *req)
  *		allocation request
  * RET SLURM_SUCCESS on success, otherwise return SLURM_ERROR with errno set
  */
-extern int slurm_het_job_will_run(List job_req_list)
+extern int slurm_het_job_will_run(list_t *job_req_list)
 {
 	job_desc_msg_t *req;
 	will_run_response_msg_t *will_run_resp;
 	char buf[256], *sep = "";
 	int rc = SLURM_SUCCESS, inx = 0;
-	ListIterator iter, itr;
+	list_itr_t *iter, *itr;
 	time_t first_start = (time_t) 0;
 	uint32_t first_job_id = 0, tot_proc_count = 0, *job_id_ptr;
-	hostset_t hs = NULL;
+	hostset_t *hs = NULL;
 	char *job_list = NULL;
 
 	if (!job_req_list || (list_count(job_req_list) == 0)) {
@@ -744,6 +743,7 @@ slurm_job_step_create (job_step_create_request_msg_t *req,
 {
 	slurm_msg_t req_msg, resp_msg;
 	int delay = 0, rc, retry = 0;
+	char *stepmgr_nodename = NULL;
 
 	slurm_msg_t_init(&req_msg);
 	slurm_msg_t_init(&resp_msg);
@@ -751,11 +751,50 @@ slurm_job_step_create (job_step_create_request_msg_t *req,
 	req_msg.data     = req;
 
 re_send:
-	if (slurm_send_recv_controller_msg(&req_msg, &resp_msg,
-					   working_cluster_rec) < 0)
+	/* xstrdup() to be consistent with reroute and be able to free. */
+	if ((stepmgr_nodename = xstrdup(getenv("SLURM_STEPMGR")))) {
+trystepmgr:
+		slurm_msg_set_r_uid(&req_msg, slurm_conf.slurmd_user_id);
+
+		if (slurm_conf_get_addr(stepmgr_nodename, &req_msg.address,
+					req_msg.flags)) {
+			/*
+			 * The node isn't in the conf, see if the
+			 * controller has an address for it.
+			 */
+			slurm_node_alias_addrs_t *alias_addrs;
+			if (!slurm_get_node_alias_addrs(stepmgr_nodename,
+							&alias_addrs)) {
+				add_remote_nodes_to_conf_tbls(
+					alias_addrs->node_list,
+					alias_addrs->node_addrs);
+			}
+			slurm_free_node_alias_addrs(alias_addrs);
+			slurm_conf_get_addr(stepmgr_nodename, &req_msg.address,
+					    req_msg.flags);
+		}
+		xfree(stepmgr_nodename);
+
+		if (slurm_send_recv_node_msg(&req_msg, &resp_msg, 0))
+			return SLURM_ERROR;
+	} else if (slurm_send_recv_controller_msg(&req_msg, &resp_msg,
+						  working_cluster_rec) < 0) {
 		return SLURM_ERROR;
+	}
 
 	switch (resp_msg.msg_type) {
+	case RESPONSE_SLURM_REROUTE_MSG:
+	{
+		reroute_msg_t *rr_msg = resp_msg.data;
+		xfree(stepmgr_nodename);
+		stepmgr_nodename = rr_msg->stepmgr;
+		rr_msg->stepmgr = NULL;
+		if (stepmgr_nodename)
+			goto trystepmgr;
+		else
+			return SLURM_ERROR;
+		break;
+	}
 	case RESPONSE_SLURM_RC:
 		rc = _handle_rc_msg(&resp_msg);
 		if ((rc < 0) && (errno == EAGAIN)) {
@@ -837,11 +876,12 @@ extern int slurm_allocation_lookup(uint32_t jobid,
  * NOTE: returns information an individual job as well
  * NOTE: free the response using list_destroy()
  */
-extern int slurm_het_job_lookup(uint32_t jobid, List *info)
+extern int slurm_het_job_lookup(uint32_t jobid, list_t **info)
 {
 	job_alloc_info_msg_t req;
 	slurm_msg_t req_msg;
 	slurm_msg_t resp_msg;
+	char *stepmgr_nodename = NULL;
 
 	memset(&req, 0, sizeof(req));
 	req.job_id = jobid;
@@ -851,9 +891,34 @@ extern int slurm_het_job_lookup(uint32_t jobid, List *info)
 	req_msg.msg_type = REQUEST_HET_JOB_ALLOC_INFO;
 	req_msg.data     = &req;
 
-	if (slurm_send_recv_controller_msg(&req_msg, &resp_msg,
-					   working_cluster_rec) < 0)
+	if ((stepmgr_nodename = xstrdup(getenv("SLURM_STEPMGR")))) {
+		slurm_msg_set_r_uid(&req_msg, slurm_conf.slurmd_user_id);
+
+		if (slurm_conf_get_addr(stepmgr_nodename, &req_msg.address,
+					req_msg.flags)) {
+			/*
+			 * The node isn't in the conf, see if the
+			 * controller has an address for it.
+			 */
+			slurm_node_alias_addrs_t *alias_addrs;
+			if (!slurm_get_node_alias_addrs(stepmgr_nodename,
+							&alias_addrs)) {
+				add_remote_nodes_to_conf_tbls(
+					alias_addrs->node_list,
+					alias_addrs->node_addrs);
+			}
+			slurm_free_node_alias_addrs(alias_addrs);
+			slurm_conf_get_addr(stepmgr_nodename, &req_msg.address,
+					    req_msg.flags);
+		}
+		xfree(stepmgr_nodename);
+
+		if (slurm_send_recv_node_msg(&req_msg, &resp_msg, 0))
+			return SLURM_ERROR;
+	} else if (slurm_send_recv_controller_msg(&req_msg, &resp_msg,
+						  working_cluster_rec) < 0) {
 		return SLURM_ERROR;
+	}
 
 	req.req_cluster = NULL;
 
@@ -864,7 +929,7 @@ extern int slurm_het_job_lookup(uint32_t jobid, List *info)
 		*info = NULL;
 		break;
 	case RESPONSE_HET_JOB_ALLOCATION:
-		*info = (List) resp_msg.data;
+		*info = resp_msg.data;
 		return SLURM_SUCCESS;
 		break;
 	default:
@@ -888,17 +953,56 @@ extern int slurm_sbcast_lookup(slurm_selected_step_t *selected_step,
 {
 	slurm_msg_t req_msg;
 	slurm_msg_t resp_msg;
+	char *stepmgr_nodename = NULL;
 
 	slurm_msg_t_init(&req_msg);
 	slurm_msg_t_init(&resp_msg);
 	req_msg.msg_type = REQUEST_JOB_SBCAST_CRED;
 	req_msg.data     = selected_step;
 
-	if (slurm_send_recv_controller_msg(&req_msg,
-					   &resp_msg,working_cluster_rec) < 0)
+trystepmgr:
+	if (stepmgr_nodename) {
+		slurm_msg_set_r_uid(&req_msg, slurm_conf.slurmd_user_id);
+
+		if (slurm_conf_get_addr(stepmgr_nodename, &req_msg.address,
+					req_msg.flags)) {
+			/*
+			 * The node isn't in the conf, see if the
+			 * controller has an address for it.
+			 */
+			slurm_node_alias_addrs_t *alias_addrs;
+			if (!slurm_get_node_alias_addrs(stepmgr_nodename,
+							&alias_addrs)) {
+				add_remote_nodes_to_conf_tbls(
+					alias_addrs->node_list,
+					alias_addrs->node_addrs);
+			}
+			slurm_free_node_alias_addrs(alias_addrs);
+			slurm_conf_get_addr(stepmgr_nodename, &req_msg.address,
+					    req_msg.flags);
+		}
+		xfree(stepmgr_nodename);
+
+		if (slurm_send_recv_node_msg(&req_msg, &resp_msg, 0))
+			return SLURM_ERROR;
+	} else if (slurm_send_recv_controller_msg(
+		    &req_msg,
+		    &resp_msg,working_cluster_rec) < 0)
 		return SLURM_ERROR;
 
 	switch (resp_msg.msg_type) {
+	case RESPONSE_SLURM_REROUTE_MSG:
+	{
+		reroute_msg_t *rr_msg = resp_msg.data;
+		stepmgr_nodename = rr_msg->stepmgr;
+		rr_msg->stepmgr = NULL;
+		if (stepmgr_nodename)
+			goto trystepmgr;
+		else
+			return SLURM_ERROR;
+
+		break;
+	}
 	case RESPONSE_SLURM_RC:
 		if (_handle_rc_msg(&resp_msg) < 0)
 			return SLURM_ERROR;
@@ -935,10 +1039,10 @@ _handle_rc_msg(slurm_msg_t *msg)
 
 /*
  * Read a Slurm hostfile specified by "filename".  "filename" must contain
- * a list of Slurm NodeNames, one per line.  Reads up to "n" number of hostnames
- * from the file. Returns a string representing a hostlist ranged string of
- * the contents of the file.  This is a helper function, it does not
- * contact any Slurm daemons.
+ * a list of Slurm NodeNames, one per line, comma seperated, or * notation.
+ * Reads up to "n" number of hostnames from the file. Returns a string
+ * representing a hostlist ranged string of the contents of the file.
+ * This is a helper function, it does not contact any Slurm daemons.
  *
  * Returns a string representing the hostlist.  Returns NULL if there are fewer
  * than "n" hostnames in the file, or if an error occurs.  If "n" ==
@@ -953,7 +1057,7 @@ char *slurm_read_hostfile(const char *filename, int n)
 	int i, j;
 	int line_size;
 	int line_num = 0;
-	hostlist_t hostlist = NULL;
+	hostlist_t *hostlist = NULL;
 	char *nodelist = NULL, *end_part = NULL;
 	char *asterisk, *tmp_text = NULL, *save_ptr = NULL, *host_name;
 	int total_file_len = 0;
@@ -1052,6 +1156,9 @@ char *slurm_read_hostfile(const char *filename, int n)
 			    (i = atoi(asterisk + 1))) {
 				asterisk[0] = '\0';
 
+				if (n != (int) NO_VAL)
+					i = MIN(i,
+						n - hostlist_count(hostlist));
 				/*
 				 * Don't forget the extra space potentially
 				 * needed
@@ -1064,6 +1171,10 @@ char *slurm_read_hostfile(const char *filename, int n)
 				hostlist_push_host(hostlist, host_name);
 			}
 			host_name = strtok_r(NULL, ",", &save_ptr);
+
+			if ((n != (int) NO_VAL) &&
+			    (hostlist_count(hostlist) == n))
+				break;
 		}
 		xfree(tmp_text);
 
@@ -1162,8 +1273,8 @@ static void _destroy_allocation_response_socket(listen_t *listen)
  * IN msg: message received
  * OUT resp: resource allocation response message or List of them
  * RET 1 if resp is filled in, 0 otherwise */
-static int
-_handle_msg(slurm_msg_t *msg, uint16_t msg_type, void **resp)
+static int _handle_msg(slurm_msg_t *msg, uint16_t msg_type, void **resp,
+		       uint32_t job_id)
 {
 	uid_t req_uid;
 	uid_t uid       = getuid();
@@ -1185,10 +1296,18 @@ _handle_msg(slurm_msg_t *msg, uint16_t msg_type, void **resp)
 		msg->data = NULL;
 		rc = 1;
 	} else if (msg->msg_type == SRUN_JOB_COMPLETE) {
-		info("Job has been cancelled");
+		srun_job_complete_msg_t *job_comp_msg = msg->data;
+		if (job_comp_msg->job_id == job_id) {
+			info("Job has been cancelled");
+		} else {
+			verbose("Ignoring SRUN_JOB_COMPLETE message for JobId=%u (our JobId=%u)",
+				job_comp_msg->job_id, job_id);
+			rc = 2;
+		}
 	} else {
-		error("%s: received spurious message type: %u",
-		      __func__, msg->msg_type);
+		error("%s: received spurious message type: %s",
+		      __func__, rpc_num2string(msg->msg_type));
+		rc = 2;
 	}
 	return rc;
 }
@@ -1198,8 +1317,8 @@ _handle_msg(slurm_msg_t *msg, uint16_t msg_type, void **resp)
  * IN msg_type: RESPONSE_RESOURCE_ALLOCATION or RESPONSE_HET_JOB_ALLOCATION
  * OUT resp: resource allocation response message or List
  * RET 1 if resp is filled in, 0 otherwise */
-static int
-_accept_msg_connection(int listen_fd, uint16_t msg_type, void **resp)
+static int _accept_msg_connection(int listen_fd, uint16_t msg_type, void **resp,
+				  uint32_t job_id)
 {
 	int	     conn_fd;
 	slurm_msg_t  *msg = NULL;
@@ -1231,7 +1350,8 @@ _accept_msg_connection(int listen_fd, uint16_t msg_type, void **resp)
 		return SLURM_ERROR;
 	}
 
-	rc = _handle_msg(msg, msg_type, resp); /* xfer payload */
+	rc = _handle_msg(msg, msg_type, resp, job_id); /* xfer payload */
+
 	slurm_free_msg(msg);
 
 	close(conn_fd);
@@ -1296,8 +1416,14 @@ static void _wait_for_allocation_response(uint32_t job_id,
 
 	info("job %u queued and waiting for resources", job_id);
 	*resp = NULL;
-	if ((rc = _wait_for_alloc_rpc(listen, timeout)) == 1)
-		rc = _accept_msg_connection(listen->fd, msg_type, resp);
+	while (true) {
+		if ((rc = _wait_for_alloc_rpc(listen, timeout)) != 1)
+			break;
+
+		if ((rc = _accept_msg_connection(listen->fd, msg_type, resp,
+						 job_id)) != 2)
+			break;
+	}
 	if (rc <= 0) {
 		errnum = errno;
 		/* Maybe the resource allocation response RPC got lost
@@ -1311,13 +1437,13 @@ static void _wait_for_allocation_response(uint32_t job_id,
 					resp) >= 0)
 				return;
 		} else if (msg_type == RESPONSE_HET_JOB_ALLOCATION) {
-			if (slurm_het_job_lookup(job_id, (List *) resp) >= 0)
+			if (slurm_het_job_lookup(job_id, (list_t **) resp) >= 0)
 				return;
 		} else {
 			error("%s: Invalid msg_type (%u)", __func__, msg_type);
 		}
 
-		if (slurm_get_errno() == ESLURM_JOB_PENDING) {
+		if (errno == ESLURM_JOB_PENDING) {
 			debug3("Still waiting for allocation");
 			errno = errnum;
 			return;
