@@ -340,6 +340,10 @@ struct job_record {
 	time_t last_sched_eval;		/* last time job was evaluated for scheduling */
 	char *licenses;			/* licenses required by the job */
 	list_t *license_list;		/* structure with license info */
+	list_t *licenses_to_preempt;    /* list of licenses the job will look
+					   for in its preemptee candidates
+					   Don't pack, don't save it's only used
+					   during resource selection */
 	char *lic_req;		/* required system licenses directly requested*/
 	acct_policy_limit_set_t limit_set; /* flags if indicate an
 					    * associated limit was set from
@@ -358,6 +362,9 @@ struct job_record {
 	bitstr_t *node_bitmap;		/* bitmap of nodes allocated to job */
 	bitstr_t *node_bitmap_cg;	/* bitmap of nodes completing job */
 	bitstr_t *node_bitmap_pr;	/* bitmap of nodes with running prolog */
+	bitstr_t *node_bitmap_preempt; /* bitmap of nodes selected for the job
+					 * when trying to preempt other jobs.
+					 * (DO NOT SAVE OR PACK). */
 	uint32_t node_cnt;		/* count of nodes currently
 					 * allocated to job */
 	uint32_t node_cnt_wag;		/* count of nodes Slurm thinks
