@@ -43,6 +43,7 @@
 
 #include <ctype.h>
 #include <errno.h>
+#include <libgen.h>
 #include <pthread.h>
 #include <stdarg.h>
 #include <stdio.h>
@@ -62,16 +63,13 @@
 
 #define XFGETS_CHUNKSIZE 64
 
-/* Static functions. */
-static size_t _xstrdup_vprintf(char **str, const char *_fmt, va_list _ap);
-
 /*
  * Define slurm-specific aliases for use by plugins, see slurm_xlator.h
  * for details.
  */
 strong_alias(_xstrcat,		slurm_xstrcat);
-strong_alias(_xstrcatat,	slurm_xstrcatat);
 strong_alias(_xstrncat,		slurm_xstrncat);
+strong_alias(_xstrncatat,	slurm_xstrncatat);
 strong_alias(_xstrcatchar,	slurm_xstrcatchar);
 strong_alias(_xstrftimecat,	slurm_xstrftimecat);
 strong_alias(_xiso8601timecat,	slurm_xiso8601timecat);
@@ -81,6 +79,7 @@ strong_alias(_xstrfmtcatat,	slurm_xstrfmtcatat);
 strong_alias(_xmemcat,		slurm_xmemcat);
 strong_alias(xstrdup,		slurm_xstrdup);
 strong_alias(xstrdup_printf,	slurm_xstrdup_printf);
+strong_alias(_xstrdup_vprintf,	slurm_xstrdup_vprintf);
 strong_alias(xstrndup,		slurm_xstrndup);
 strong_alias(xbasename,		slurm_xbasename);
 strong_alias(xdirname,		slurm_xdirname);
@@ -96,6 +95,7 @@ strong_alias(xstrcasecmp,       slurm_xstrcasecmp);
 strong_alias(xstrncasecmp,      slurm_xstrncasecmp);
 strong_alias(xstrstr,           slurm_xstrstr);
 strong_alias(xstrcasestr,       slurm_xstrcasestr);
+strong_alias(xbase64_from_base64url, slurm_xbase64_from_base64url);
 
 /*
  * Ensure that a string has enough space to add 'needed' characters.
@@ -146,29 +146,29 @@ void _xstrcat(char **str1, const char *str2)
  * Append str2 onto str at pos, * expanding buf as needed. pos is updated to the
  * end of the appended string.
  *
- * Meant to be used in loops contructing longer strings that are performance
+ * Meant to be used in loops constructing longer strings that are performance
  * sensitive, as xstrcat() needs to re-seek to the end of str making the string
  * construction worse by another O(log(strlen)) factor.
  */
-void _xstrcatat(char **str, char **pos, const char *str2)
+void _xstrncatat(char **str, char **pos, const char *str2, ssize_t append_len)
 {
-	size_t orig_len, append_len;
+	size_t orig_len;
 
 	if (!str2)
 		return;
 
-	append_len = strlen(str2);
+	if (append_len < 0)
+		append_len = strlen(str2);
 
 	/* No string yet to append to, so return a copy of str2. */
 	if (!*str) {
-		*str = xstrdup(str2);
+		*str = xstrndup(str2, append_len);
 		*pos = *str + append_len;
 		return;
 	}
 
 	if (!*pos) {
 		orig_len = strlen(*str);
-		*pos = *str + orig_len;
 	} else {
 		xassert(*pos >= *str);
 		orig_len = *pos - *str;
@@ -361,7 +361,7 @@ void _xstrfmtcat(char **str, const char *fmt, ...)
  * expanding buf as needed. pos is updated to the end of the appended
  * string.
  *
- * Meant to be used in loops contructing longer strings that are performance
+ * Meant to be used in loops constructing longer strings that are performance
  * sensitive, as xstrfmtcat() needs to re-seek to the end of str making the
  * string construction worse by another O(log(strlen)) factor.
  */
@@ -387,7 +387,6 @@ void _xstrfmtcatat(char **str, char **pos, const char *fmt, ...)
 
 	if (!*pos) {
 		orig_len = strlen(*str);
-		*pos = *str + orig_len;
 	} else {
 		xassert(*pos >= *str);
 		orig_len = *pos - *str;
@@ -443,36 +442,28 @@ char * xbasename(char *path)
 }
 
 /*
- * Specialized dirname implementation which returns the result of removing the
- * basename to the given path, or a "." (dot) if the given path doesn't contain
- * a slash.
+ * Slurm's safe implementation of dirname.
  *
- * NOTE: This implementation differs from the libgen/libc ones, and it does not
- *	 conform to the XPG 4.2 or any other standard. For instance, given
- *	 "/tmp/" as an argument, a conformant implementation would return "/",
- *	 but this will return "/tmp".
+ * This function returns a malloc'ed string which lets the caller to safely
+ * manipulate it, unlike in dirname() which in some cases returns pointers
+ * to the passed string, and in other cases a new malloc'ed string. dirname()
+ * can also modify the original string so it is preferred to pas a copy of the
+ * input string.
  *
- * NOTE: This doesn't handle multiple and contiguous slashes.
+ * Besides that, the result is exactly like in dirname().
  *
- * IN:	char pointer to a path
- * RET:	the path without the xbasename or a dot if no slashes
+ * IN path - path to get the dirname from.
+ * RETURN res - string containing the path without the basename.
  */
 char *xdirname(const char *path)
 {
+	char *res = NULL;
 	char *fname = xstrdup(path);
-	char *slash;
 
-	if (!fname)
-		return xstrdup(".");
+	res = xstrdup(dirname(fname));
+	xfree(fname);
 
-	if (!(slash = strrchr(fname, '/'))) {
-		xfree(fname);
-		return xstrdup(".");
-	}
-
-	*slash = '\0';
-
-	return fname;
+	return res;
 }
 
 /*
@@ -660,19 +651,21 @@ bool xstring_is_whitespace(const char *str)
 	return true;
 }
 
-/*
- * If str make everything lowercase.  Should not be called on static char *'s
- */
-char *xstrtolower(char *str)
+extern bool xstrtolower(char *str)
 {
-	if (str) {
-		int j = 0;
-		while (str[j]) {
-			str[j] = tolower((int)str[j]);
-			j++;
-		}
+	bool modified = false;
+
+	if (!str)
+		return modified;
+
+	for (int j = 0; str[j]; j++) {
+		int c = tolower((int) str[j]);
+
+		if (c != str[j])
+			modified = true;
+		str[j] = c;
 	}
-	return str;
+	return modified;
 }
 
 /* safe strchr */
@@ -784,7 +777,7 @@ char *xstrcasestr(const char *haystack, const char *needle)
  *   fmt (IN)		format of string and args if any
  *   RETURN		copy of formated string
  */
-static size_t _xstrdup_vprintf(char **str, const char *fmt, va_list ap)
+size_t _xstrdup_vprintf(char **str, const char *fmt, va_list ap)
 {
 	/* Start out with a size of 100 bytes. */
 	int n, size = 100;
@@ -898,3 +891,34 @@ extern char *xstring_bytes2printable(const unsigned char *string, int len,
 
 	return str;
 }
+
+extern char *xbase64_from_base64url(const char *in)
+{
+	char *out;
+	int i;
+
+	/* extra padding in case the padding was stripped off */
+	out = xmalloc(strlen(in) + 3);
+
+	for (i = 0; i < strlen(in); i++) {
+		switch (in[i]) {
+		case '-':
+			out[i] = '+';
+			break;
+		case '_':
+			out[i] = '/';
+			break;
+		default:
+			out[i] = in[i];
+		}
+	}
+
+	/*
+	 * Fix padding in case it was trimmed.
+	 * String length must be a multiple of 4.
+	 */
+	for (int padding = 4 - (i % 4); padding < 4 && padding; padding--)
+		out[i++] = '=';
+
+	return out;
+}
