@@ -1,8 +1,7 @@
 /*****************************************************************************\
  *  cgroup.c - driver for cgroup plugin
  *****************************************************************************
- *  Copyright (C) 2021 SchedMD LLC
- *  Written by Felip Moll <felip.moll@schedmd.com>
+ *  Copyright (C) SchedMD LLC.
  *
  *  This file is part of Slurm, a resource management program.
  *  For details, see <https://slurm.schedmd.com/>.
@@ -68,7 +67,7 @@ typedef struct {
         int	(*constrain_apply)	(cgroup_ctl_type_t sub,
                                          cgroup_level_t level,
                                          uint32_t task_id);
-	int	(*step_start_oom_mgr)	(void);
+	int	(*step_start_oom_mgr)	(stepd_step_rec_t *step);
 	cgroup_oom_t *(*step_stop_oom_mgr) (stepd_step_rec_t *step);
 	int	(*task_addto)		(cgroup_ctl_type_t sub,
 					 stepd_step_rec_t *step, pid_t pid,
@@ -76,6 +75,8 @@ typedef struct {
 	cgroup_acct_t *(*task_get_acct_data) (uint32_t taskid);
 	long int (*get_acct_units)	(void);
 	bool (*has_feature) (cgroup_ctl_feature_t f);
+	char *(*get_scope_path)(void);
+	int (*setup_scope)(char *scope_path);
 } slurm_ops_t;
 
 /*
@@ -103,12 +104,15 @@ static const char *syms[] = {
 	"cgroup_p_task_get_acct_data",
 	"cgroup_p_get_acct_units",
 	"cgroup_p_has_feature",
+	"cgroup_p_get_scope_path",
+	"cgroup_p_setup_scope",
 };
 
 /* Local variables */
 static slurm_ops_t ops;
 static plugin_context_t *g_context = NULL;
 static pthread_mutex_t g_context_lock =	PTHREAD_MUTEX_INITIALIZER;
+static plugin_init_t plugin_inited = PLUGIN_NOT_INITED;
 
 cgroup_conf_t slurm_cgroup_conf;
 
@@ -117,6 +121,8 @@ static buf_t *cg_conf_buf = NULL;
 static bool cg_conf_inited = false;
 static bool cg_conf_exist = true;
 
+static char scope_path[PATH_MAX] = "";
+
 /* local functions */
 static void _cgroup_conf_fini();
 static void _clear_slurm_cgroup_conf();
@@ -125,6 +131,15 @@ static int _unpack_cgroup_conf(buf_t *buffer);
 static void _read_slurm_cgroup_conf(void);
 
 /* Local functions */
+static int _defunct_option(void **dest, slurm_parser_enum_t type,
+			  const char *key, const char *value,
+			  const char *line, char **leftover)
+{
+	error_in_daemon("The option \"%s\" is defunct, please remove it from cgroup.conf.",
+			key);
+	return 0;
+}
+
 static void _cgroup_conf_fini()
 {
 	slurm_rwlock_wrlock(&cg_conf_lock);
@@ -149,10 +164,8 @@ static void _init_slurm_cgroup_conf(void)
 {
 	_clear_slurm_cgroup_conf();
 
-	slurm_cgroup_conf.allowed_kmem_space = -1;
 	slurm_cgroup_conf.allowed_ram_space = 100;
 	slurm_cgroup_conf.allowed_swap_space = 0;
-	slurm_cgroup_conf.cgroup_automount = false;
 	slurm_cgroup_conf.cgroup_mountpoint = xstrdup(DEFAULT_CGROUP_BASEDIR);
 	slurm_cgroup_conf.cgroup_plugin = xstrdup(DEFAULT_CGROUP_PLUGIN);
 #ifndef MULTIPLE_SLURMD
@@ -162,19 +175,17 @@ static void _init_slurm_cgroup_conf(void)
 #endif
 	slurm_cgroup_conf.constrain_cores = false;
 	slurm_cgroup_conf.constrain_devices = false;
-	slurm_cgroup_conf.constrain_kmem_space = false;
 	slurm_cgroup_conf.constrain_ram_space = false;
 	slurm_cgroup_conf.constrain_swap_space = false;
 	slurm_cgroup_conf.enable_controllers = false;
 	slurm_cgroup_conf.ignore_systemd = false;
 	slurm_cgroup_conf.ignore_systemd_on_failure = false;
-	slurm_cgroup_conf.max_kmem_percent = 100;
 	slurm_cgroup_conf.max_ram_percent = 100;
 	slurm_cgroup_conf.max_swap_percent = 100;
 	slurm_cgroup_conf.memory_swappiness = NO_VAL64;
-	slurm_cgroup_conf.min_kmem_space = XCGROUP_DEFAULT_MIN_RAM;
 	slurm_cgroup_conf.min_ram_space = XCGROUP_DEFAULT_MIN_RAM;
-	slurm_cgroup_conf.root_owned_cgroups = true;
+	slurm_cgroup_conf.signal_children_processes = false;
+	slurm_cgroup_conf.systemd_timeout = 1000;
 }
 
 static void _pack_cgroup_conf(buf_t *buffer)
@@ -189,7 +200,6 @@ static void _pack_cgroup_conf(buf_t *buffer)
 		return;
 	}
 	packbool(1, buffer);
-	packbool(slurm_cgroup_conf.cgroup_automount, buffer);
 	packstr(slurm_cgroup_conf.cgroup_mountpoint, buffer);
 
 	packstr(slurm_cgroup_conf.cgroup_prepend, buffer);
@@ -202,11 +212,6 @@ static void _pack_cgroup_conf(buf_t *buffer)
 
 	pack64(slurm_cgroup_conf.min_ram_space, buffer);
 
-	packbool(slurm_cgroup_conf.constrain_kmem_space, buffer);
-	packfloat(slurm_cgroup_conf.allowed_kmem_space, buffer);
-	packfloat(slurm_cgroup_conf.max_kmem_percent, buffer);
-	pack64(slurm_cgroup_conf.min_kmem_space, buffer);
-
 	packbool(slurm_cgroup_conf.constrain_swap_space, buffer);
 	packfloat(slurm_cgroup_conf.allowed_swap_space, buffer);
 	packfloat(slurm_cgroup_conf.max_swap_percent, buffer);
@@ -218,13 +223,13 @@ static void _pack_cgroup_conf(buf_t *buffer)
 	packbool(slurm_cgroup_conf.ignore_systemd, buffer);
 	packbool(slurm_cgroup_conf.ignore_systemd_on_failure, buffer);
 
-	packbool(slurm_cgroup_conf.root_owned_cgroups, buffer);
 	packbool(slurm_cgroup_conf.enable_controllers, buffer);
+	packbool(slurm_cgroup_conf.signal_children_processes, buffer);
+	pack64(slurm_cgroup_conf.systemd_timeout, buffer);
 }
 
 static int _unpack_cgroup_conf(buf_t *buffer)
 {
-	uint32_t uint32_tmp = 0;
 	bool tmpbool = false;
 	/*
 	 * No protocol version needed, at the time of writing we are only
@@ -236,12 +241,11 @@ static int _unpack_cgroup_conf(buf_t *buffer)
 		return SLURM_SUCCESS;
 	}
 
-	safe_unpackbool(&slurm_cgroup_conf.cgroup_automount, buffer);
-	safe_unpackstr_xmalloc(&slurm_cgroup_conf.cgroup_mountpoint,
-			       &uint32_tmp, buffer);
+	_clear_slurm_cgroup_conf();
 
-	safe_unpackstr_xmalloc(&slurm_cgroup_conf.cgroup_prepend,
-			       &uint32_tmp, buffer);
+	safe_unpackstr(&slurm_cgroup_conf.cgroup_mountpoint, buffer);
+
+	safe_unpackstr(&slurm_cgroup_conf.cgroup_prepend, buffer);
 
 	safe_unpackbool(&slurm_cgroup_conf.constrain_cores, buffer);
 
@@ -251,25 +255,21 @@ static int _unpack_cgroup_conf(buf_t *buffer)
 
 	safe_unpack64(&slurm_cgroup_conf.min_ram_space, buffer);
 
-	safe_unpackbool(&slurm_cgroup_conf.constrain_kmem_space, buffer);
-	safe_unpackfloat(&slurm_cgroup_conf.allowed_kmem_space, buffer);
-	safe_unpackfloat(&slurm_cgroup_conf.max_kmem_percent, buffer);
-	safe_unpack64(&slurm_cgroup_conf.min_kmem_space, buffer);
-
 	safe_unpackbool(&slurm_cgroup_conf.constrain_swap_space, buffer);
 	safe_unpackfloat(&slurm_cgroup_conf.allowed_swap_space, buffer);
 	safe_unpackfloat(&slurm_cgroup_conf.max_swap_percent, buffer);
 	safe_unpack64(&slurm_cgroup_conf.memory_swappiness, buffer);
 
 	safe_unpackbool(&slurm_cgroup_conf.constrain_devices, buffer);
-	safe_unpackstr_xmalloc(&slurm_cgroup_conf.cgroup_plugin,
-			       &uint32_tmp, buffer);
+	safe_unpackstr(&slurm_cgroup_conf.cgroup_plugin, buffer);
 
 	safe_unpackbool(&slurm_cgroup_conf.ignore_systemd, buffer);
 	safe_unpackbool(&slurm_cgroup_conf.ignore_systemd_on_failure, buffer);
 
-	safe_unpackbool(&slurm_cgroup_conf.root_owned_cgroups, buffer);
 	safe_unpackbool(&slurm_cgroup_conf.enable_controllers, buffer);
+	safe_unpackbool(&slurm_cgroup_conf.signal_children_processes, buffer);
+	safe_unpack64(&slurm_cgroup_conf.systemd_timeout, buffer);
+
 	return SLURM_SUCCESS;
 
 unpack_error:
@@ -285,7 +285,7 @@ unpack_error:
 static void _read_slurm_cgroup_conf(void)
 {
 	s_p_options_t options[] = {
-		{"CgroupAutomount", S_P_BOOLEAN},
+		{"CgroupAutomount", S_P_BOOLEAN, _defunct_option},
 		{"CgroupMountpoint", S_P_STRING},
 		{"CgroupReleaseAgentDir", S_P_STRING},
 		{"ConstrainCores", S_P_BOOLEAN},
@@ -294,10 +294,6 @@ static void _read_slurm_cgroup_conf(void)
 		{"MaxRAMPercent", S_P_FLOAT},
 		{"MinRAMSpace", S_P_UINT64},
 		{"ConstrainSwapSpace", S_P_BOOLEAN},
-		{"ConstrainKmemSpace", S_P_BOOLEAN},
-		{"AllowedKmemSpace", S_P_FLOAT},
-		{"MaxKmemPercent", S_P_FLOAT},
-		{"MinKmemSpace", S_P_UINT64},
 		{"AllowedSwapSpace", S_P_FLOAT},
 		{"MaxSwapPercent", S_P_FLOAT},
 		{"MemoryLimitEnforcement", S_P_BOOLEAN},
@@ -308,35 +304,32 @@ static void _read_slurm_cgroup_conf(void)
 		{"CgroupPlugin", S_P_STRING},
 		{"IgnoreSystemd", S_P_BOOLEAN},
 		{"IgnoreSystemdOnFailure", S_P_BOOLEAN},
-		{"RootOwnedCgroups", S_P_BOOLEAN},
 		{"EnableControllers", S_P_BOOLEAN},
+		{"SignalChildrenProcesses", S_P_BOOLEAN},
+		{"SystemdTimeout", S_P_UINT64},
 		{NULL} };
 	s_p_hashtbl_t *tbl = NULL;
 	char *conf_path = NULL, *tmp_str;
 	struct stat buf;
 	size_t sz;
-	bool kmem_deprecate_msg = false;
 
 	/* Get the cgroup.conf path and validate the file */
 	conf_path = get_extra_conf_path("cgroup.conf");
 	if ((conf_path == NULL) || (stat(conf_path, &buf) == -1)) {
-		log_flag(CGROUP, "%s: No cgroup.conf file (%s)", __func__,
-			 conf_path);
+		info("%s: No cgroup.conf file (%s), using defaults",
+		     __func__, conf_path);
 		cg_conf_exist = false;
 	} else {
 		debug("Reading cgroup.conf file %s", conf_path);
 
 		tbl = s_p_hashtbl_create(options);
-		if (s_p_parse_file(tbl, NULL, conf_path, false, NULL, false) ==
+		if (s_p_parse_file(tbl, NULL, conf_path, 0, NULL) ==
 		    SLURM_ERROR) {
 			fatal("Could not open/read/parse cgroup.conf file %s",
 			      conf_path);
 		}
 
 		/* cgroup initialization parameters */
-		(void) s_p_get_boolean(&slurm_cgroup_conf.cgroup_automount,
-				       "CgroupAutomount", tbl);
-
 		if (s_p_get_string(&tmp_str, "CgroupMountpoint", tbl)) {
 			/* Remove the trailing / if any. */
 			sz = strlen(tmp_str);
@@ -368,36 +361,6 @@ static void _read_slurm_cgroup_conf(void)
 		(void) s_p_get_boolean(&slurm_cgroup_conf.constrain_swap_space,
 				       "ConstrainSwapSpace", tbl);
 
-		/*
-		 * Disable constrain_kmem_space by default because of a known
-		 * bug in Linux kernel version 3, early versions of kernel
-		 * version 4, and RedHat/CentOS 6 and 7, which leaks slab
-		 * caches, eventually causing the machine to be unable to create
-		 * new cgroups.
-		 *
-		 * kmem.limit_in_bytes is deprecated in the linux kernel
-		 * and is not used in cgroup v2, so we are deprecating
-		 * ConstrainKmemSpace and related parameters.
-		 */
-		if (s_p_get_boolean(&slurm_cgroup_conf.constrain_kmem_space,
-				    "ConstrainKmemSpace", tbl))
-			kmem_deprecate_msg = true;
-
-		if (s_p_get_float(&slurm_cgroup_conf.allowed_kmem_space,
-				  "AllowedKmemSpace", tbl))
-			kmem_deprecate_msg = true;
-
-		if (s_p_get_float(&slurm_cgroup_conf.max_kmem_percent,
-				  "MaxKmemPercent", tbl))
-			kmem_deprecate_msg = true;
-
-		if (s_p_get_uint64(&slurm_cgroup_conf.min_kmem_space,
-				   "MinKmemSpace", tbl))
-			kmem_deprecate_msg = true;
-
-		if (kmem_deprecate_msg && running_in_daemon())
-			error("AllowedKmemSpace, ConstrainKmemSpace, MaxKmemPercent, and MinKmemSpace are deprecated, and will be removed in a future release");
-
 		(void) s_p_get_float(&slurm_cgroup_conf.allowed_swap_space,
 				     "AllowedSwapSpace", tbl);
 
@@ -442,11 +405,14 @@ static void _read_slurm_cgroup_conf(void)
 			    "IgnoreSystemdOnFailure", tbl)))
 			slurm_cgroup_conf.ignore_systemd_on_failure = false;
 
-		(void) s_p_get_boolean(&slurm_cgroup_conf.root_owned_cgroups,
-				       "RootOwnedCgroups", tbl);
-
 		(void) s_p_get_boolean(&slurm_cgroup_conf.enable_controllers,
 				       "EnableControllers", tbl);
+		(void) s_p_get_boolean(
+			&slurm_cgroup_conf.signal_children_processes,
+			"SignalChildrenProcesses", tbl);
+
+		(void) s_p_get_uint64(&slurm_cgroup_conf.systemd_timeout,
+				      "SystemdTimeout", tbl);
 
 		s_p_hashtbl_destroy(tbl);
 	}
@@ -459,6 +425,7 @@ static void _read_slurm_cgroup_conf(void)
 /* Autodetect logic inspired from systemd source code */
 extern char *autodetect_cgroup_version(void)
 {
+#ifdef WITH_CGROUP
 	struct statfs fs;
 	int cgroup_ver = -1;
 
@@ -508,6 +475,8 @@ extern char *autodetect_cgroup_version(void)
 		error("unsupported cgroup version %d", cgroup_ver);
 		break;
 	}
+#endif
+
 	return NULL;
 }
 
@@ -526,13 +495,16 @@ extern int cgroup_conf_init(void)
 	if (!cg_conf_inited) {
 		_init_slurm_cgroup_conf();
 		_read_slurm_cgroup_conf();
-		/*
-		 * Initialize and pack cgroup.conf info into a buffer that can
-		 * be used by slurmd to send to stepd every time, instead of
-		 * re-packing every time we want to send to slurmstepd
-		 */
-		cg_conf_buf = init_buf(0);
-		_pack_cgroup_conf(cg_conf_buf);
+		if (running_in_slurmd()) {
+			/*
+			 * Initialize and pack cgroup.conf info into a buffer
+			 * that can be used by slurmd to send to stepd every
+			 * time, instead of re-packing every time we want to
+			 * send to slurmstepd
+			 */
+			cg_conf_buf = init_buf(0);
+			_pack_cgroup_conf(cg_conf_buf);
+		}
 		cg_conf_inited = true;
 	} else
 		rc = SLURM_ERROR;
@@ -547,12 +519,6 @@ extern void cgroup_conf_destroy(void)
 	_cgroup_conf_fini();
 }
 
-extern void cgroup_conf_reinit(void)
-{
-	cgroup_conf_destroy();
-	cgroup_conf_init();
-}
-
 extern void cgroup_free_limits(cgroup_limits_t *limits)
 {
 	if (!limits)
@@ -576,7 +542,6 @@ extern void cgroup_init_limits(cgroup_limits_t *limits)
 	limits->device.minor = NO_VAL;
 	limits->limit_in_bytes = NO_VAL64;
 	limits->soft_limit_in_bytes = NO_VAL64;
-	limits->kmem_limit_in_bytes = NO_VAL64;
 	limits->memsw_limit_in_bytes = NO_VAL64;
 	limits->swappiness = NO_VAL64;
 }
@@ -586,144 +551,108 @@ extern void cgroup_init_limits(cgroup_limits_t *limits)
  *      cgroup.conf  file and return a key pair <name,value> ordered list.
  * RET List with cgroup.conf <name,value> pairs if no error, NULL otherwise.
  */
-extern List cgroup_get_conf_list(void)
+extern list_t *cgroup_get_conf_list(void)
 {
-	config_key_pair_t *key_pair;
-	List cgroup_conf_l;
+	list_t *cgroup_conf_l;
 	cgroup_conf_t *cg_conf = &slurm_cgroup_conf;
 
 	xassert(cg_conf_inited);
 
-	slurm_rwlock_rdlock(&cg_conf_lock);
-
-	/* Fill list with cgroup config key pairs */
 	cgroup_conf_l = list_create(destroy_config_key_pair);
 
-	key_pair = xmalloc(sizeof(config_key_pair_t));
-	key_pair->name = xstrdup("CgroupAutomount");
-	key_pair->value = xstrdup_printf("%s", cg_conf->cgroup_automount ?
-					 "yes" : "no");
-	list_append(cgroup_conf_l, key_pair);
+	slurm_rwlock_rdlock(&cg_conf_lock);
 
-	key_pair = xmalloc(sizeof(config_key_pair_t));
-	key_pair->name = xstrdup("CgroupMountpoint");
-	key_pair->value = xstrdup(cg_conf->cgroup_mountpoint);
-	list_append(cgroup_conf_l, key_pair);
+	add_key_pair(cgroup_conf_l, "CgroupMountpoint", "%s",
+		     cg_conf->cgroup_mountpoint);
+	add_key_pair_bool(cgroup_conf_l, "ConstrainCores",
+			  cg_conf->constrain_cores);
+	add_key_pair_bool(cgroup_conf_l, "ConstrainRAMSpace",
+			  cg_conf->constrain_ram_space);
+	add_key_pair(cgroup_conf_l, "AllowedRAMSpace", "%.1f%%",
+		     cg_conf->allowed_ram_space);
+	add_key_pair(cgroup_conf_l, "MaxRAMPercent", "%.1f%%",
+		     cg_conf->max_ram_percent);
+	add_key_pair(cgroup_conf_l, "MinRAMSpace", "%"PRIu64"MB",
+		     cg_conf->min_ram_space);
+	add_key_pair_bool(cgroup_conf_l, "ConstrainSwapSpace",
+			  cg_conf->constrain_swap_space);
+	add_key_pair(cgroup_conf_l, "AllowedSwapSpace", "%.1f%%",
+		     cg_conf->allowed_swap_space);
+	add_key_pair(cgroup_conf_l, "MaxSwapPercent", "%.1f%%",
+		     cg_conf->max_swap_percent);
+	add_key_pair_bool(cgroup_conf_l, "ConstrainDevices",
+			  cg_conf->constrain_devices);
+	add_key_pair(cgroup_conf_l, "CgroupPlugin", "%s",
+		     cg_conf->cgroup_plugin);
+	add_key_pair_bool(cgroup_conf_l, "IgnoreSystemd",
+			  cg_conf->ignore_systemd);
+	add_key_pair_bool(cgroup_conf_l, "IgnoreSystemdOnFailure",
+			  cg_conf->ignore_systemd_on_failure);
+	add_key_pair_bool(cgroup_conf_l, "EnableControllers",
+			  cg_conf->enable_controllers);
 
-	key_pair = xmalloc(sizeof(config_key_pair_t));
-	key_pair->name = xstrdup("ConstrainCores");
-	key_pair->value = xstrdup_printf("%s", cg_conf->constrain_cores ?
-					 "yes" : "no");
-	list_append(cgroup_conf_l, key_pair);
-
-	key_pair = xmalloc(sizeof(config_key_pair_t));
-	key_pair->name = xstrdup("ConstrainRAMSpace");
-	key_pair->value = xstrdup_printf("%s", cg_conf->constrain_ram_space ?
-					 "yes" : "no");
-	list_append(cgroup_conf_l, key_pair);
-
-	key_pair = xmalloc(sizeof(config_key_pair_t));
-	key_pair->name = xstrdup("AllowedRAMSpace");
-	key_pair->value = xstrdup_printf("%.1f%%", cg_conf->allowed_ram_space);
-	list_append(cgroup_conf_l, key_pair);
-
-	key_pair = xmalloc(sizeof(config_key_pair_t));
-	key_pair->name = xstrdup("MaxRAMPercent");
-	key_pair->value = xstrdup_printf("%.1f%%", cg_conf->max_ram_percent);
-	list_append(cgroup_conf_l, key_pair);
-
-	key_pair = xmalloc(sizeof(config_key_pair_t));
-	key_pair->name = xstrdup("MinRAMSpace");
-	key_pair->value = xstrdup_printf("%"PRIu64" MB",
-					 cg_conf->min_ram_space);
-	list_append(cgroup_conf_l, key_pair);
-
-	key_pair = xmalloc(sizeof(config_key_pair_t));
-	key_pair->name = xstrdup("ConstrainSwapSpace");
-	key_pair->value = xstrdup_printf("%s", cg_conf->constrain_swap_space ?
-					 "yes" : "no");
-	list_append(cgroup_conf_l, key_pair);
-
-	key_pair = xmalloc(sizeof(config_key_pair_t));
-	key_pair->name = xstrdup("ConstrainKmemSpace");
-	key_pair->value = xstrdup_printf("%s", cg_conf->constrain_kmem_space ?
-					 "yes" : "no");
-	list_append(cgroup_conf_l, key_pair);
-
-	key_pair = xmalloc(sizeof(config_key_pair_t));
-	key_pair->name = xstrdup("AllowedKmemSpace");
-	if (cg_conf->allowed_kmem_space >= 0)
-		key_pair->value = xstrdup_printf("%.0f Bytes",
-						 cg_conf->allowed_kmem_space);
-	list_append(cgroup_conf_l, key_pair);
-
-	key_pair = xmalloc(sizeof(config_key_pair_t));
-	key_pair->name = xstrdup("MaxKmemPercent");
-	key_pair->value = xstrdup_printf("%.1f%%", cg_conf->max_kmem_percent);
-	list_append(cgroup_conf_l, key_pair);
-
-	key_pair = xmalloc(sizeof(config_key_pair_t));
-	key_pair->name = xstrdup("MinKmemSpace");
-	key_pair->value = xstrdup_printf("%"PRIu64" MB",
-					 cg_conf->min_kmem_space);
-	list_append(cgroup_conf_l, key_pair);
-
-	key_pair = xmalloc(sizeof(config_key_pair_t));
-	key_pair->name = xstrdup("AllowedSwapSpace");
-	key_pair->value = xstrdup_printf("%.1f%%", cg_conf->allowed_swap_space);
-	list_append(cgroup_conf_l, key_pair);
-
-	key_pair = xmalloc(sizeof(config_key_pair_t));
-	key_pair->name = xstrdup("MaxSwapPercent");
-	key_pair->value = xstrdup_printf("%.1f%%", cg_conf->max_swap_percent);
-	list_append(cgroup_conf_l, key_pair);
-
-	key_pair = xmalloc(sizeof(config_key_pair_t));
-	key_pair->name = xstrdup("ConstrainDevices");
-	key_pair->value = xstrdup_printf("%s", cg_conf->constrain_devices ?
-					 "yes" : "no");
-	list_append(cgroup_conf_l, key_pair);
-
-	key_pair = xmalloc(sizeof(config_key_pair_t));
-	key_pair->name = xstrdup("MemorySwappiness");
 	if (cg_conf->memory_swappiness != NO_VAL64)
-		key_pair->value = xstrdup_printf("%"PRIu64,
-						 cg_conf->memory_swappiness);
-	list_append(cgroup_conf_l, key_pair);
+		add_key_pair(cgroup_conf_l, "MemorySwappiness", "%"PRIu64,
+			     cg_conf->memory_swappiness);
+	else
+		add_key_pair(cgroup_conf_l, "MemorySwappiness", "(null)");
 
-	key_pair = xmalloc(sizeof(config_key_pair_t));
-	key_pair->name = xstrdup("CgroupPlugin");
-	key_pair->value = xstrdup(cg_conf->cgroup_plugin);
-	list_append(cgroup_conf_l, key_pair);
-
-	key_pair = xmalloc(sizeof(config_key_pair_t));
-	key_pair->name = xstrdup("IgnoreSystemd");
-	key_pair->value = xstrdup_printf("%s",
-					 cg_conf->ignore_systemd ?
-					 "yes" : "no");
-	list_append(cgroup_conf_l, key_pair);
-
-	key_pair = xmalloc(sizeof(config_key_pair_t));
-	key_pair->name = xstrdup("IgnoreSystemdOnFailure");
-	key_pair->value = xstrdup_printf("%s",
-					 cg_conf->ignore_systemd_on_failure ?
-					 "yes" : "no");
-	list_append(cgroup_conf_l, key_pair);
-
-	key_pair = xmalloc(sizeof(config_key_pair_t));
-	key_pair->name = xstrdup("EnableControllers");
-	key_pair->value = xstrdup_printf("%s",
-					 cg_conf->enable_controllers ?
-					 "yes" : "no");
-	list_append(cgroup_conf_l, key_pair);
-
-	list_sort(cgroup_conf_l, (ListCmpF) sort_key_pairs);
+	add_key_pair(cgroup_conf_l, "SystemdTimeout", "%"PRIu64" ms",
+		     cg_conf->systemd_timeout);
 
 	slurm_rwlock_unlock(&cg_conf_lock);
 
+	list_sort(cgroup_conf_l, (ListCmpF) sort_key_pairs);
+
 	return cgroup_conf_l;
 }
 
+/*
+ * This function is called from slurmd to send the cgroup state (at present
+ * only the scope path in cgroup/v2) to the recently forked slurmstepd, since
+ * slurmstepd might not be able to infer the correct scope path when we are
+ * running into a container.
+ */
+extern int cgroup_write_state(int fd)
+{
+	int len = 0;
+	char *step_path = NULL;
+
+	if (plugin_inited == PLUGIN_INITED) {
+		step_path = (*(ops.get_scope_path))();
+		if (step_path)
+			len = strlen(step_path) + 1;
+	}
+
+	safe_write(fd, &len, sizeof(int));
+	if (step_path)
+		safe_write(fd, step_path, len);
+
+	return SLURM_SUCCESS;
+rwfail:
+	return SLURM_ERROR;
+}
+
+/*
+ * This function is called from slurmstepd before the cgroup plugin is
+ * initialized. It records the cgroup plugin state passed from slurmd
+ * (at present only the scope path in cgroup/v2) in this slurmstepd so it
+ * can be later used by the plugin when it is initialized.
+ */
+extern int cgroup_read_state(int fd)
+{
+	int len;
+
+	safe_read(fd, &len, sizeof(int));
+
+	if (len)
+		safe_read(fd, scope_path, len);
+
+	return SLURM_SUCCESS;
+rwfail:
+	return SLURM_ERROR;
+}
+
 extern int cgroup_write_conf(int fd)
 {
 	int len;
@@ -749,9 +678,6 @@ extern int cgroup_read_conf(int fd)
 
 	slurm_rwlock_wrlock(&cg_conf_lock);
 
-	if (cg_conf_inited)
-		_clear_slurm_cgroup_conf();
-
 	safe_read(fd, &len, sizeof(int));
 	buffer = init_buf(len);
 	safe_read(fd, buffer->head, len);
@@ -783,9 +709,10 @@ extern bool cgroup_memcg_job_confinement(void)
 	/* read cgroup configuration */
 	slurm_rwlock_rdlock(&cg_conf_lock);
 
-	if ((slurm_cgroup_conf.constrain_ram_space ||
-	     slurm_cgroup_conf.constrain_swap_space) &&
-	    xstrstr(slurm_conf.task_plugin, "cgroup"))
+	if (xstrcmp(slurm_cgroup_conf.cgroup_plugin, "disabled") &&
+	    ((slurm_cgroup_conf.constrain_ram_space ||
+	      slurm_cgroup_conf.constrain_swap_space) &&
+	     xstrstr(slurm_conf.task_plugin, "cgroup")))
 		status = true;
 
 	slurm_rwlock_unlock(&cg_conf_lock);
@@ -806,12 +733,19 @@ extern int cgroup_g_init(void)
 
 	slurm_mutex_lock(&g_context_lock);
 
-	if (g_context)
+	if (plugin_inited)
 		goto done;
 
-	cgroup_conf_init();
+	if (cgroup_conf_init() != SLURM_SUCCESS)
+		log_flag(CGROUP, "cgroup conf was already initialized.");
+
 	type = slurm_cgroup_conf.cgroup_plugin;
 
+	if (!xstrcmp(type, "disabled")) {
+		plugin_inited = PLUGIN_NOOP;
+		goto done;
+	}
+
 	if (!xstrcmp(type, "autodetect")) {
 		if (!(type = autodetect_cgroup_version())) {
 			rc = SLURM_ERROR;
@@ -825,9 +759,21 @@ extern int cgroup_g_init(void)
 	if (!g_context) {
 		error("cannot create %s context for %s", plugin_type, type);
 		rc = SLURM_ERROR;
+		plugin_inited = PLUGIN_NOT_INITED;
 		goto done;
 	}
 
+	/*
+	 * We have recorded the scope_path here previously, configure it now in
+	 * the plugin.
+	 */
+	rc = (*(ops.setup_scope))(scope_path);
+	if (rc == SLURM_ERROR) {
+		error("cannot setup the scope for %s", plugin_type);
+		goto done;
+	}
+
+	plugin_inited = PLUGIN_INITED;
 done:
 	slurm_mutex_unlock(&g_context_lock);
 
@@ -836,94 +782,131 @@ done:
 
 extern int cgroup_g_fini(void)
 {
-	int rc;
-
-	if (!g_context)
-		return SLURM_SUCCESS;
+	int rc = SLURM_SUCCESS;
 
 	slurm_mutex_lock(&g_context_lock);
-	rc = plugin_context_destroy(g_context);
-	g_context = NULL;
-	slurm_mutex_unlock(&g_context_lock);
+	if (g_context) {
+		rc = plugin_context_destroy(g_context);
+		g_context = NULL;
+	}
 
 	cgroup_conf_destroy();
+	plugin_inited = PLUGIN_NOT_INITED;
 
+	slurm_mutex_unlock(&g_context_lock);
 	return rc;
 }
 
 extern int cgroup_g_initialize(cgroup_ctl_type_t sub)
 {
-	xassert(g_context);
+	xassert(plugin_inited != PLUGIN_NOT_INITED);
+
+	if (plugin_inited == PLUGIN_NOOP)
+		fatal("%s: Trying to initialize cgroups but CgroupPlugin=disabled is set in cgroup.conf. Please, unset any configuration that is using cgroups.",
+		      __func__);
 
 	return (*(ops.initialize))(sub);
 }
 
 extern int cgroup_g_system_create(cgroup_ctl_type_t sub)
 {
-	xassert(g_context);
+	xassert(plugin_inited != PLUGIN_NOT_INITED);
+
+	if (plugin_inited == PLUGIN_NOOP)
+		return SLURM_SUCCESS;
 
 	return (*(ops.system_create))(sub);
 }
 
 extern int cgroup_g_system_addto(cgroup_ctl_type_t sub, pid_t *pids, int npids)
 {
-	xassert(g_context);
+	xassert(plugin_inited != PLUGIN_NOT_INITED);
+
+	if (plugin_inited == PLUGIN_NOOP)
+		return SLURM_SUCCESS;
 
 	return (*(ops.system_addto))(sub, pids, npids);
 }
 
 extern int cgroup_g_system_destroy(cgroup_ctl_type_t sub)
 {
-	xassert(g_context);
+	xassert(plugin_inited != PLUGIN_NOT_INITED);
+
+	if (plugin_inited == PLUGIN_NOOP)
+		return SLURM_SUCCESS;
 
 	return (*(ops.system_destroy))(sub);
 }
 
 extern int cgroup_g_step_create(cgroup_ctl_type_t sub, stepd_step_rec_t *step)
 {
-	xassert(g_context);
+	xassert(plugin_inited != PLUGIN_NOT_INITED);
+
+	if (plugin_inited == PLUGIN_NOOP)
+		return SLURM_SUCCESS;
 
 	return (*(ops.step_create))(sub, step);
 }
 
 extern int cgroup_g_step_addto(cgroup_ctl_type_t sub, pid_t *pids, int npids)
 {
-	xassert(g_context);
+	xassert(plugin_inited != PLUGIN_NOT_INITED);
+
+	if (plugin_inited == PLUGIN_NOOP)
+		return SLURM_SUCCESS;
 
 	return (*(ops.step_addto))(sub, pids, npids);
 }
 
 extern int cgroup_g_step_get_pids(pid_t **pids, int *npids)
 {
-	xassert(g_context);
+	xassert(plugin_inited != PLUGIN_NOT_INITED);
+
+	if (plugin_inited == PLUGIN_NOOP) {
+		*npids = 0;
+		*pids = NULL;
+		return SLURM_SUCCESS;
+	}
 
 	return (*(ops.step_get_pids))(pids, npids);
 }
 
 extern int cgroup_g_step_suspend(void)
 {
-	xassert(g_context);
+	xassert(plugin_inited != PLUGIN_NOT_INITED);
+
+	if (plugin_inited == PLUGIN_NOOP)
+		return SLURM_SUCCESS;
 
 	return (*(ops.step_suspend))();
 }
 
 extern int cgroup_g_step_resume(void)
 {
-	xassert(g_context);
+	xassert(plugin_inited != PLUGIN_NOT_INITED);
+
+	if (plugin_inited == PLUGIN_NOOP)
+		return SLURM_SUCCESS;
 
 	return (*(ops.step_resume))();
 }
 
 extern int cgroup_g_step_destroy(cgroup_ctl_type_t sub)
 {
-	xassert(g_context);
+	xassert(plugin_inited != PLUGIN_NOT_INITED);
+
+	if (plugin_inited == PLUGIN_NOOP)
+		return SLURM_SUCCESS;
 
 	return (*(ops.step_destroy))(sub);
 }
 
 extern bool cgroup_g_has_pid(pid_t pid)
 {
-	xassert(g_context);
+	xassert(plugin_inited != PLUGIN_NOT_INITED);
+
+	if (plugin_inited == PLUGIN_NOOP)
+		return false;
 
 	return (*(ops.has_pid))(pid);
 }
@@ -931,7 +914,10 @@ extern bool cgroup_g_has_pid(pid_t pid)
 extern cgroup_limits_t *cgroup_g_constrain_get(cgroup_ctl_type_t sub,
 					       cgroup_level_t level)
 {
-	xassert(g_context);
+	xassert(plugin_inited != PLUGIN_NOT_INITED);
+
+	if (plugin_inited == PLUGIN_NOOP)
+		return NULL;
 
 	return (*(ops.constrain_get))(sub, level);
 }
@@ -939,7 +925,10 @@ extern cgroup_limits_t *cgroup_g_constrain_get(cgroup_ctl_type_t sub,
 extern int cgroup_g_constrain_set(cgroup_ctl_type_t sub, cgroup_level_t level,
 				  cgroup_limits_t *limits)
 {
-	xassert(g_context);
+	xassert(plugin_inited != PLUGIN_NOT_INITED);
+
+	if (plugin_inited == PLUGIN_NOOP)
+		return SLURM_SUCCESS;
 
 	return (*(ops.constrain_set))(sub, level, limits);
 }
@@ -947,21 +936,32 @@ extern int cgroup_g_constrain_set(cgroup_ctl_type_t sub, cgroup_level_t level,
 extern int cgroup_g_constrain_apply(cgroup_ctl_type_t sub, cgroup_level_t level,
                                     uint32_t task_id)
 {
-	xassert(g_context);
+	xassert(plugin_inited != PLUGIN_NOT_INITED);
+
+	if (plugin_inited == PLUGIN_NOOP)
+		return SLURM_SUCCESS;
 
 	return (*(ops.constrain_apply))(sub, level, task_id);
 }
 
-extern int cgroup_g_step_start_oom_mgr()
+extern int cgroup_g_step_start_oom_mgr(stepd_step_rec_t *step)
 {
-	xassert(g_context);
+	xassert(plugin_inited != PLUGIN_NOT_INITED);
 
-	return (*(ops.step_start_oom_mgr))();
+	if (plugin_inited == PLUGIN_NOOP)
+		return SLURM_SUCCESS;
+
+	return (*(ops.step_start_oom_mgr))(step);
 }
 
 extern cgroup_oom_t *cgroup_g_step_stop_oom_mgr(stepd_step_rec_t *step)
 {
-	xassert(g_context);
+	xassert(plugin_inited != PLUGIN_NOT_INITED);
+
+	if (plugin_inited == PLUGIN_NOOP) {
+		cgroup_oom_t *empty_oom = xmalloc(sizeof(*empty_oom));
+		return empty_oom;
+	}
 
 	return (*(ops.step_stop_oom_mgr))(step);
 }
@@ -969,28 +969,42 @@ extern cgroup_oom_t *cgroup_g_step_stop_oom_mgr(stepd_step_rec_t *step)
 extern int cgroup_g_task_addto(cgroup_ctl_type_t sub, stepd_step_rec_t *step,
 			       pid_t pid, uint32_t task_id)
 {
-	xassert(g_context);
+	xassert(plugin_inited != PLUGIN_NOT_INITED);
+
+	if (plugin_inited == PLUGIN_NOOP)
+		return SLURM_SUCCESS;
 
 	return (*(ops.task_addto))(sub, step, pid, task_id);
 }
 
 extern cgroup_acct_t *cgroup_g_task_get_acct_data(uint32_t taskid)
 {
-	xassert(g_context);
+	xassert(plugin_inited != PLUGIN_NOT_INITED);
+
+	if (plugin_inited == PLUGIN_NOOP) {
+		cgroup_acct_t *empty_acct = xmalloc(sizeof(*empty_acct));
+		return empty_acct;
+	}
 
 	return (*(ops.task_get_acct_data))(taskid);
 }
 
-extern long int cgroup_g_get_acct_units()
+extern long int cgroup_g_get_acct_units(void)
 {
-	xassert(g_context);
+	xassert(plugin_inited != PLUGIN_NOT_INITED);
+
+	if (plugin_inited == PLUGIN_NOOP)
+		return (long int)USEC_IN_SEC;
 
 	return (*(ops.get_acct_units))();
 }
 
 extern bool cgroup_g_has_feature(cgroup_ctl_feature_t f)
 {
-	xassert(g_context);
+	xassert(plugin_inited != PLUGIN_NOT_INITED);
+
+	if (plugin_inited == PLUGIN_NOOP)
+		return false;
 
 	return (*(ops.has_feature))(f);
 }
