@@ -47,7 +47,7 @@ static void _setup_res_cond(slurmdb_res_cond_t *res_cond,
 			    char **extra)
 {
 	int set = 0;
-	ListIterator itr = NULL;
+	list_itr_t *itr = NULL;
 	char *object = NULL;
 
 	if (!res_cond) {
@@ -158,7 +158,7 @@ static void _setup_res_cond(slurmdb_res_cond_t *res_cond,
 
 static int _setup_clus_res_cond(slurmdb_res_cond_t *res_cond, char **extra)
 {
-	ListIterator itr;
+	list_itr_t *itr;
 	bool set = 0;
 	char *tmp = NULL;
 	int query_clusters = 0;
@@ -428,7 +428,7 @@ static int _fill_in_res_rec(mysql_conn_t *mysql_conn, slurmdb_res_rec_t *res)
 }
 
 static int _add_res(mysql_conn_t *mysql_conn, slurmdb_res_rec_t *object,
-		    char *user_name, int *added, ListIterator itr_in)
+		    char *user_name, int *added, list_itr_t *itr_in)
 {
 	char *cols = NULL, *extra = NULL, *vals = NULL, *query = NULL,
 		*tmp_extra = NULL;
@@ -523,7 +523,7 @@ static int _add_clus_res(mysql_conn_t *mysql_conn, slurmdb_res_rec_t *res,
 	time_t now = time(NULL);
 	int rc = SLURM_SUCCESS, cluster_cnt;
 	slurmdb_clus_res_rec_t *object;
-	ListIterator itr;
+	list_itr_t *itr;
 	uint32_t total_pos = 100;
 	char *percent_str = "%";
 
@@ -633,10 +633,10 @@ static int _add_clus_res(mysql_conn_t *mysql_conn, slurmdb_res_rec_t *res,
 	return rc;
 }
 
-static List _get_clus_res(mysql_conn_t *mysql_conn, uint32_t res_id,
-			  char *extra)
+static list_t *_get_clus_res(mysql_conn_t *mysql_conn, uint32_t res_id,
+			     char *extra)
 {
-	List ret_list;
+	list_t *ret_list;
 	char *query = NULL, *tmp = NULL;
 	MYSQL_RES *result = NULL;
 	MYSQL_ROW row;
@@ -694,9 +694,9 @@ static List _get_clus_res(mysql_conn_t *mysql_conn, uint32_t res_id,
 }
 
 extern int as_mysql_add_res(mysql_conn_t *mysql_conn, uint32_t uid,
-			    List res_list)
+			    list_t *res_list)
 {
-	ListIterator itr = NULL;
+	list_itr_t *itr = NULL;
 	int rc = SLURM_SUCCESS;
 	slurmdb_res_rec_t *object = NULL;
 	char *user_name = NULL;
@@ -757,14 +757,14 @@ extern int as_mysql_add_res(mysql_conn_t *mysql_conn, uint32_t uid,
 	return rc;
 }
 
-extern List as_mysql_get_res(mysql_conn_t *mysql_conn, uid_t uid,
-			     slurmdb_res_cond_t *res_cond)
+extern list_t *as_mysql_get_res(mysql_conn_t *mysql_conn, uid_t uid,
+				slurmdb_res_cond_t *res_cond)
 {
 	char *query = NULL;
 	char *extra = NULL;
 	char *clus_extra = NULL;
 	char *tmp = NULL;
-	List res_list = NULL;
+	list_t *res_list = NULL;
 	int i=0;
 	MYSQL_RES *result = NULL;
 	MYSQL_ROW row;
@@ -833,7 +833,7 @@ extern List as_mysql_get_res(mysql_conn_t *mysql_conn, uid_t uid,
 	res_list = list_create(slurmdb_destroy_res_rec);
 	while ((row = mysql_fetch_row(result))) {
 		uint32_t id = 0;
-		List clus_res_list = NULL;
+		list_t *clus_res_list = NULL;
 		slurmdb_res_rec_t *res;
 
 		if (row[RES_REQ_ID] && row[RES_REQ_ID][0])
@@ -893,10 +893,10 @@ extern List as_mysql_get_res(mysql_conn_t *mysql_conn, uid_t uid,
 	return res_list;
 }
 
-extern List as_mysql_remove_res(mysql_conn_t *mysql_conn, uint32_t uid,
-				slurmdb_res_cond_t *res_cond)
+extern list_t *as_mysql_remove_res(mysql_conn_t *mysql_conn, uint32_t uid,
+				   slurmdb_res_cond_t *res_cond)
 {
-	List ret_list = NULL;
+	list_t *ret_list = NULL;
 	char *name_char = NULL, *clus_char = NULL;
 	char *user_name = NULL;
 	char *query = NULL, *extra = NULL, *clus_extra = NULL;
@@ -1044,11 +1044,11 @@ extern List as_mysql_remove_res(mysql_conn_t *mysql_conn, uint32_t uid,
 	return ret_list;
 }
 
-extern List as_mysql_modify_res(mysql_conn_t *mysql_conn, uint32_t uid,
-				slurmdb_res_cond_t *res_cond,
-				slurmdb_res_rec_t *res)
+extern list_t *as_mysql_modify_res(mysql_conn_t *mysql_conn, uint32_t uid,
+				   slurmdb_res_cond_t *res_cond,
+				   slurmdb_res_rec_t *res)
 {
-	List ret_list = NULL;
+	list_t *ret_list = NULL;
 	char *vals = NULL, *clus_vals = NULL;
 	time_t now = time(NULL);
 	char *user_name = NULL, *tmp = NULL, *col_names = NULL;
@@ -1203,8 +1203,8 @@ extern List as_mysql_modify_res(mysql_conn_t *mysql_conn, uint32_t uid,
 		if ((res->flags & SLURMDB_RES_FLAG_ABSOLUTE) &&
 		    (res->flags & SLURMDB_RES_FLAG_REMOVE)) {
 		} else if ((res->flags & SLURMDB_RES_FLAG_ABSOLUTE) ||
-		    (slurm_atoul(row[RES_REQ_FLAGS]) &
-		     SLURMDB_RES_FLAG_ABSOLUTE)) {
+			   (slurm_atoul(row[RES_REQ_FLAGS]) &
+			    SLURMDB_RES_FLAG_ABSOLUTE)) {
 			total_pos = slurm_atoul(row[RES_REQ_COUNT]);
 			percent_str = "";
 		}
