@@ -1,7 +1,7 @@
 /****************************************************************************\
  *  slurmdbd_agent.c - functions to the agent talking to the SlurmDBD
  *****************************************************************************
- *  Copyright (C) 2011-2018 SchedMD LLC.
+ *  Copyright (C) SchedMD LLC.
  *  Copyright (C) 2008-2010 Lawrence Livermore National Security.
  *  Produced at Lawrence Livermore National Laboratory (cf, DISCLAIMER).
  *  Written by Morris Jette <jette1@llnl.gov>
@@ -50,7 +50,12 @@ enum {
 	MAX_DBD_ACTION_EXIT
 };
 
-slurm_persist_conn_t *slurmdbd_conn = NULL;
+typedef struct {
+	uint32_t msg_size;
+	list_t *my_list;
+} foreach_get_my_list_t;
+
+persist_conn_t *slurmdbd_conn = NULL;
 
 
 #define DBD_MAGIC		0xDEAD3219
@@ -60,7 +65,7 @@ slurm_persist_conn_t *slurmdbd_conn = NULL;
 static pthread_mutex_t agent_lock = PTHREAD_MUTEX_INITIALIZER;
 static pthread_cond_t  agent_cond = PTHREAD_COND_INITIALIZER;
 static pthread_cond_t shutdown_cond = PTHREAD_COND_INITIALIZER;
-static List      agent_list     = (List) NULL;
+static list_t *agent_list = NULL;
 static pthread_t agent_tid      = 0;
 
 static bool      halt_agent          = 0;
@@ -72,7 +77,79 @@ static pthread_cond_t  slurmdbd_cond = PTHREAD_COND_INITIALIZER;
 
 static int max_dbd_msg_action = MAX_DBD_DEFAULT_ACTION;
 
-static int _unpack_return_code(uint16_t rpc_version, buf_t *buffer)
+typedef struct {
+	list_t *id_rc_list;
+	int rc;
+} rc_msg_t;
+
+extern int jobacct_storage_p_job_heavy(void *db_conn, job_record_t *job_ptr);
+
+static int _sending_script_env(void *x, void *args)
+{
+	dbd_id_rc_msg_t *id_ptr = x;
+	job_record_t *job_ptr;
+
+	xassert(id_ptr);
+
+	if (!(job_ptr = find_job_record(id_ptr->job_id)))
+		return 0;
+
+	xassert(job_ptr);
+	xassert(job_ptr->details);
+
+	if ((slurm_conf.conf_flags & CONF_FLAG_SJS) &&
+	    (id_ptr->flags & JOB_SEND_SCRIPT) &&
+	    job_ptr->details->script_hash)
+		job_ptr->bit_flags |= JOB_SEND_SCRIPT;
+	if ((slurm_conf.conf_flags & CONF_FLAG_SJE) &&
+	    (id_ptr->flags & JOB_SEND_ENV) &&
+	    job_ptr->details->env_hash)
+		job_ptr->bit_flags |= JOB_SEND_ENV;
+
+	if (jobacct_storage_p_job_heavy(slurmdbd_conn, job_ptr) ==
+	    SLURM_SUCCESS) {
+		job_ptr->bit_flags &= ~JOB_SEND_SCRIPT;
+		job_ptr->bit_flags &= ~JOB_SEND_ENV;
+	}
+
+	return 0;
+}
+
+static bool _add_sending_script_env(dbd_id_rc_msg_t *id_ptr, rc_msg_t *rc_msg)
+{
+	xassert(id_ptr);
+
+	if (!(id_ptr->flags & (JOB_SEND_SCRIPT | JOB_SEND_ENV)))
+		return false;
+	/*
+	 * We are in the agent_lock here, we can't call
+	 * jobacct_storage_p_job_heavy() which will call slurmdbd_agent_send()
+	 * creating deadlock. Add message to a list to handle things later.
+	 */
+	if (!rc_msg->id_rc_list)
+		rc_msg->id_rc_list = list_create(slurmdbd_free_id_rc_msg);
+	list_append(rc_msg->id_rc_list, id_ptr);
+
+	return true;
+}
+
+static void _process_id_rc_list(list_t *id_rc_list)
+{
+	slurmctld_lock_t job_write_lock = {
+		.job = WRITE_LOCK,
+	};
+
+	if (!id_rc_list)
+		return;
+
+	lock_slurmctld(job_write_lock);
+	(void) list_for_each(id_rc_list, _sending_script_env, NULL);
+	unlock_slurmctld(job_write_lock);
+	FREE_NULL_LIST(id_rc_list);
+}
+
+static int _unpack_return_code(uint16_t rpc_version, buf_t *buffer,
+			       rc_msg_t *rc_msg)
 {
 	uint16_t msg_type = -1;
 	persist_rc_msg_t *msg;
@@ -80,6 +157,8 @@ static int _unpack_return_code(uint16_t rpc_version, buf_t *buffer)
 	persist_msg_t resp;
 	int rc = SLURM_ERROR;
 
+	xassert(rc_msg);
+
 	memset(&resp, 0, sizeof(persist_msg_t));
 	if ((rc = unpack_slurmdbd_msg(&resp, slurmdbd_conn->version, buffer))
 	    != SLURM_SUCCESS) {
@@ -95,8 +174,8 @@ static int _unpack_return_code(uint16_t rpc_version, buf_t *buffer)
 		log_flag(PROTOCOL, "msg_type:DBD_ID_RC return_code:%s JobId=%u db_index=%"PRIu64,
 			 slurm_strerror(rc), id_msg->job_id,
 			 id_msg->db_index);
-
-		slurmdbd_free_id_rc_msg(id_msg);
+		if (!_add_sending_script_env(id_msg, rc_msg))
+			slurmdbd_free_id_rc_msg(id_msg);
 		if (rc != SLURM_SUCCESS)
 			error("DBD_ID_RC is %d", rc);
 		break;
@@ -141,19 +220,39 @@ static int _unpack_return_code(uint16_t rpc_version, buf_t *buffer)
 	return rc;
 }
 
-static int _get_return_code(void)
+static int _get_return_code(rc_msg_t *rc_msg)
 {
 	int rc = SLURM_ERROR;
 	buf_t *buffer = slurm_persist_recv_msg(slurmdbd_conn);
 	if (buffer == NULL)
 		return rc;
 
-	rc = _unpack_return_code(slurmdbd_conn->version, buffer);
+	rc = _unpack_return_code(slurmdbd_conn->version, buffer, rc_msg);
 
 	FREE_NULL_BUFFER(buffer);
 	return rc;
 }
 
+static int _get_return_codes(void *x, void *arg)
+{
+	buf_t *out_buf = x;
+	rc_msg_t *rc_msg = arg;
+	buf_t *b;
+
+	if ((rc_msg->rc = _unpack_return_code(
+		     slurmdbd_conn->version, out_buf, rc_msg)) !=
+	    SLURM_SUCCESS)
+		return -1;
+
+	if ((b = list_dequeue(agent_list))) {
+		FREE_NULL_BUFFER(b);
+	} else {
+		error("DBD_GOT_MULT_MSG unpack message error");
+	}
+
+	return 0;
+}
+
 static int _handle_mult_rc_ret(void)
 {
 	buf_t *buffer;
@@ -161,7 +260,7 @@ static int _handle_mult_rc_ret(void)
 	persist_rc_msg_t *msg = NULL;
 	dbd_list_msg_t *list_msg = NULL;
 	int rc = SLURM_ERROR;
-	buf_t *out_buf = NULL;
+	rc_msg_t rc_msg = { 0 };
 
 	buffer = slurm_persist_recv_msg(slurmdbd_conn);
 	if (buffer == NULL)
@@ -180,25 +279,14 @@ static int _handle_mult_rc_ret(void)
 
 		slurm_mutex_lock(&agent_lock);
 		if (agent_list) {
-			ListIterator itr =
-				list_iterator_create(list_msg->my_list);
-			while ((out_buf = list_next(itr))) {
-				buf_t *b;
-				if ((rc = _unpack_return_code(
-					     slurmdbd_conn->version, out_buf))
-				    != SLURM_SUCCESS)
-					break;
-
-				if ((b = list_dequeue(agent_list))) {
-					FREE_NULL_BUFFER(b);
-				} else {
-					error("DBD_GOT_MULT_MSG "
-					      "unpack message error");
-				}
-			}
-			list_iterator_destroy(itr);
+			list_for_each(list_msg->my_list, _get_return_codes,
+				      &rc_msg);
 		}
 		slurm_mutex_unlock(&agent_lock);
+		rc = rc_msg.rc;
+
+		_process_id_rc_list(rc_msg.id_rc_list);
+
 		slurmdbd_free_list_msg(list_msg);
 		break;
 	case PERSIST_RC:
@@ -314,7 +402,6 @@ static void _load_dbd_state(void)
 			      dbd_fname);
 	} else {
 		char *ver_str = NULL;
-		uint32_t ver_str_len;
 
 		buffer = _load_dbd_rec(fd);
 		if (buffer == NULL)
@@ -322,7 +409,7 @@ static void _load_dbd_state(void)
 		/* This is set to the end of the buffer for send so we
 		   need to set it back to 0 */
 		set_buf_offset(buffer, 0);
-		safe_unpackstr_xmalloc(&ver_str, &ver_str_len, buffer);
+		safe_unpackstr(&ver_str, buffer);
 		debug3("Version string in dbd_state header is %s", ver_str);
 	unpack_error:
 		FREE_NULL_BUFFER(buffer);
@@ -529,13 +616,6 @@ static void _max_dbd_msg_action(uint32_t *msg_cnt)
 		*msg_cnt -= purged;
 		info("purge %d step records", purged);
 	}
-	if (*msg_cnt >= (slurm_conf.max_dbd_msgs - 1)) {
-		uint16_t purge_type = DBD_JOB_START;
-		purged = list_delete_all(agent_list, _purge_agent_list_req,
-					 &purge_type);
-		*msg_cnt -= purged;
-		info("purge %d job start records", purged);
-	}
 }
 
 static int _print_agent_list_msg_type(void *x, void *arg)
@@ -584,6 +664,19 @@ static void _print_agent_list_msg_types(void)
 	xfree(mlist);
 }
 
+static int _get_my_list(void *x, void *arg)
+{
+	buf_t *buffer = x;
+	foreach_get_my_list_t *args = arg;
+
+	args->msg_size += size_buf(buffer);
+	if (args->msg_size > MAX_MSG_SIZE)
+		return -1;
+	list_enqueue(args->my_list, buffer);
+
+	return 0;
+}
+
 static void *_agent(void *x)
 {
 	int rc;
@@ -604,14 +697,14 @@ static void *_agent(void *x)
 	list_req.data = &list_msg;
 	memset(&list_msg, 0, sizeof(dbd_list_msg_t));
 
-	log_flag(AGENT, "slurmdbd agent_count=%d with msg_type=%s",
+	log_flag(DBD_AGENT, "slurmdbd agent_count=%d with msg_type=%s",
 		 list_count(agent_list),
 		 slurmdbd_msg_type_2_str(list_req.msg_type, 1));
 
 	while (*slurmdbd_conn->shutdown == 0) {
 		slurm_mutex_lock(&slurmdbd_lock);
 		if (halt_agent) {
-			log_flag(AGENT, "slurmdbd agent halt with agent_count=%d",
+			log_flag(DBD_AGENT, "slurmdbd agent halt with agent_count=%d",
 				 list_count(agent_list));
 
 			slurm_cond_wait(&slurmdbd_cond, &slurmdbd_lock);
@@ -625,7 +718,7 @@ static void *_agent(void *x)
 			if (slurmdbd_conn->fd < 0) {
 				fail_time = time(NULL);
 
-				log_flag(AGENT, "slurmdbd disconnected with agent_count=%d",
+				log_flag(DBD_AGENT, "slurmdbd disconnected with agent_count=%d",
 					 list_count(agent_list));
 			}
 		}
@@ -650,26 +743,21 @@ static void *_agent(void *x)
 			slurm_mutex_unlock(&agent_lock);
 			continue;
 		} else if (((cnt > 0) && ((cnt % 100) == 0)) ||
-		           (slurm_conf.debug_flags & DEBUG_FLAG_AGENT))
+		           (slurm_conf.debug_flags & DEBUG_FLAG_DBD_AGENT))
 			info("agent_count:%d", cnt);
 		/* Leave item on the queue until processing complete */
 		if (agent_list) {
-			uint32_t msg_size = sizeof(list_req);
 			if (cnt > 1) {
-				int agent_count = 0;
-				ListIterator agent_itr =
-					list_iterator_create(agent_list);
-				list_msg.my_list = list_create(NULL);
-				while ((buffer = list_next(agent_itr))) {
-					msg_size += size_buf(buffer);
-					if (msg_size > MAX_MSG_SIZE)
-						break;
-					list_enqueue(list_msg.my_list, buffer);
-					agent_count++;
-					if (agent_count > 1000)
-						break;
-				}
-				list_iterator_destroy(agent_itr);
+				int max_rpcs = 1000;
+				foreach_get_my_list_t args = {
+					.msg_size = sizeof(list_req),
+					.my_list = list_create(NULL),
+				};
+
+				list_msg.my_list = args.my_list;
+
+				list_for_each_max(agent_list, &max_rpcs,
+						  _get_my_list, &args, 1, true);
 				buffer = pack_slurmdbd_msg(
 					&list_req, SLURM_PROTOCOL_VERSION);
 			} else
@@ -704,7 +792,12 @@ static void *_agent(void *x)
 		} else if (list_msg.my_list) {
 			rc = _handle_mult_rc_ret();
 		} else {
-			rc = _get_return_code();
+			rc_msg_t rc_msg = { 0 };
+
+			rc = _get_return_code(&rc_msg);
+
+			_process_id_rc_list(rc_msg.id_rc_list);
+
 			if (rc == EAGAIN) {
 				if (*slurmdbd_conn->shutdown) {
 					slurm_mutex_unlock(&slurmdbd_lock);
@@ -749,7 +842,7 @@ static void *_agent(void *x)
 
 			fail_time = time(NULL);
 
-			if (slurm_conf.debug_flags & DEBUG_FLAG_AGENT) {
+			if (slurm_conf.debug_flags & DEBUG_FLAG_DBD_AGENT) {
 				info("slurmdbd agent failed with rc:%d",
 				     rc);
 				_print_agent_list_msg_types();
@@ -800,15 +893,14 @@ static void _shutdown_agent(void)
 	if (agent_running)
 		slurm_cond_broadcast(&agent_cond);
 	slurm_mutex_unlock(&agent_lock);
-	pthread_join(agent_tid,  NULL);
-	agent_tid = 0;
+	slurm_thread_join(agent_tid);
 }
 
 /****************************************************************************
  * Socket open/close/read/write functions
  ****************************************************************************/
 
-extern void slurmdbd_agent_set_conn(slurm_persist_conn_t *pc)
+extern void slurmdbd_agent_set_conn(persist_conn_t *pc)
 {
 	if (!running_in_slurmctld())
 		return;
