@@ -10,7 +10,7 @@
  *  Portions Copyright (C) 2013 Bull S. A. S.
  *		Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois.
  *
- *  Portions Copyright (C) 2013 SchedMD LLC.
+ *  Copyright (C) SchedMD LLC.
  *
  *  This file is part of Slurm, a resource management program.
  *  For details, see <http://www.schedmd.com/slurmdocs/>.
@@ -55,19 +55,20 @@
 #include <inttypes.h>
 #include <unistd.h>
 #include <math.h>
-#include <curl/curl.h>
 
 #include "src/common/slurm_xlator.h"
 #include "src/common/fd.h"
 #include "src/common/macros.h"
-#include "src/common/slurm_acct_gather_profile.h"
+#include "src/interfaces/acct_gather_profile.h"
 #include "src/common/slurm_protocol_api.h"
 #include "src/common/slurm_protocol_defs.h"
 #include "src/common/slurm_time.h"
 #include "src/common/timers.h"
 #include "src/common/xstring.h"
-#include "src/slurmd/common/proctrack.h"
+#include "src/curl/slurm_curl.h"
+#include "src/interfaces/proctrack.h"
 
+#define DEFAULT_INFLUXDB_TIMEOUT 10
 
 /*
  * These variables are required by the generic plugin interface.  If they
@@ -104,6 +105,7 @@ typedef struct {
 	uint32_t def;
 	char *password;
 	char *rt_policy;
+	uint32_t timeout;
 	char *username;
 } slurm_influxdb_conf_t;
 
@@ -114,12 +116,6 @@ typedef struct {
 	char * name;
 } table_t;
 
-/* Type for handling HTTP responses */
-struct http_response {
-	char *message;
-	size_t size;
-};
-
 union data_t{
 	uint64_t u;
 	double	 d;
@@ -173,34 +169,12 @@ static uint32_t _determine_profile(void)
 	return profile;
 }
 
-/* Callback to handle the HTTP response */
-static size_t _write_callback(void *contents, size_t size, size_t nmemb,
-			      void *userp)
-{
-	size_t realsize = size * nmemb;
-	struct http_response *mem = (struct http_response *) userp;
-
-	debug3("%s %s called", plugin_type, __func__);
-
-	mem->message = xrealloc(mem->message, mem->size + realsize + 1);
-
-	memcpy(&(mem->message[mem->size]), contents, realsize);
-	mem->size += realsize;
-	mem->message[mem->size] = 0;
-
-	return realsize;
-}
-
 /* Try to send data to influxdb */
 static int _send_data(const char *data)
 {
-	CURL *curl_handle = NULL;
-	CURLcode res;
-	struct http_response chunk;
 	int rc = SLURM_SUCCESS;
-	long response_code;
-	static int error_cnt = 0;
-	char *url = NULL;
+	long response_code = 0;
+	char *url = NULL, *response_str;
 	size_t length;
 
 	debug3("%s %s called", plugin_type, __func__);
@@ -222,53 +196,13 @@ static int _send_data(const char *data)
 		return rc;
 	}
 
-	DEF_TIMERS;
-	START_TIMER;
-
-	if (curl_global_init(CURL_GLOBAL_ALL) != 0) {
-		error("%s %s: curl_global_init: %m", plugin_type, __func__);
-		rc = SLURM_ERROR;
-		goto cleanup_global_init;
-	} else if ((curl_handle = curl_easy_init()) == NULL) {
-		error("%s %s: curl_easy_init: %m", plugin_type, __func__);
-		rc = SLURM_ERROR;
-		goto cleanup_easy_init;
-	}
-
 	xstrfmtcat(url, "%s/write?db=%s&rp=%s&precision=s", influxdb_conf.host,
 		   influxdb_conf.database, influxdb_conf.rt_policy);
-
-	chunk.message = xmalloc(1);
-	chunk.size = 0;
-
-	curl_easy_setopt(curl_handle, CURLOPT_URL, url);
-	if (influxdb_conf.password)
-		curl_easy_setopt(curl_handle, CURLOPT_PASSWORD,
-				 influxdb_conf.password);
-	curl_easy_setopt(curl_handle, CURLOPT_POST, 1);
-	curl_easy_setopt(curl_handle, CURLOPT_POSTFIELDS, datastr);
-	curl_easy_setopt(curl_handle, CURLOPT_POSTFIELDSIZE, strlen(datastr));
-	if (influxdb_conf.username)
-		curl_easy_setopt(curl_handle, CURLOPT_USERNAME,
-				 influxdb_conf.username);
-	curl_easy_setopt(curl_handle, CURLOPT_WRITEFUNCTION, _write_callback);
-	curl_easy_setopt(curl_handle, CURLOPT_WRITEDATA, (void *) &chunk);
-
-	if ((res = curl_easy_perform(curl_handle)) != CURLE_OK) {
-		if ((error_cnt++ % 100) == 0)
-			error("%s %s: curl_easy_perform failed to send data (discarded). Reason: %s",
-			      plugin_type, __func__, curl_easy_strerror(res));
-		rc = SLURM_ERROR;
-		goto cleanup;
-	}
-
-	if ((res = curl_easy_getinfo(curl_handle, CURLINFO_RESPONSE_CODE,
-				     &response_code)) != CURLE_OK) {
-		error("%s %s: curl_easy_getinfo response code failed: %s",
-		      plugin_type, __func__, curl_easy_strerror(res));
-		rc = SLURM_ERROR;
-		goto cleanup;
-	}
+	rc = slurm_curl_request(datastr, url, influxdb_conf.username,
+				influxdb_conf.password, false,
+				influxdb_conf.timeout, &response_str,
+				&response_code, HTTP_REQUEST_POST, true);
+	xfree(url);
 
 	/* In general, status codes of the form 2xx indicate success,
 	 * 4xx indicate that InfluxDB could not understand the request, and
@@ -276,34 +210,24 @@ static int _send_data(const char *data)
 	 * Errors are returned in JSON.
 	 * https://docs.influxdata.com/influxdb/v0.13/concepts/api/
 	 */
-	if (response_code >= 200 && response_code <= 205) {
+	if (rc != SLURM_SUCCESS) {
+		error("send data failed");
+	} else if ((response_code >= 200) && (response_code <= 205)) {
 		debug2("%s %s: data write success", plugin_type, __func__);
-		if (error_cnt > 0)
-			error_cnt = 0;
 	} else {
 		rc = SLURM_ERROR;
 		debug2("%s %s: data write failed, response code: %ld",
 		       plugin_type, __func__, response_code);
 		if (slurm_conf.debug_flags & DEBUG_FLAG_PROFILE) {
 			/* Strip any trailing newlines. */
-			while (chunk.message[strlen(chunk.message) - 1] == '\n')
-				chunk.message[strlen(chunk.message) - 1] = '\0';
+			while (response_str[strlen(response_str) - 1] == '\n')
+				response_str[strlen(response_str) - 1] = '\0';
 			info("%s %s: JSON response body: %s", plugin_type,
-			     __func__, chunk.message);
+			     __func__, response_str);
 		}
 	}
 
-cleanup:
-	xfree(chunk.message);
-	xfree(url);
-cleanup_easy_init:
-	curl_easy_cleanup(curl_handle);
-cleanup_global_init:
-	curl_global_cleanup();
-
-	END_TIMER;
-	log_flag(PROFILE, "%s %s: took %s to send data",
-		 plugin_type, __func__, TIME_STR);
+	xfree(response_str);
 
 	if (data) {
 		datastr = xstrdup(data);
@@ -327,6 +251,9 @@ extern int init(void)
 	if (!running_in_slurmstepd())
 		return SLURM_SUCCESS;
 
+	if (slurm_curl_init())
+		return SLURM_ERROR;
+
 	datastr = xmalloc(BUF_SIZE);
 	return SLURM_SUCCESS;
 }
@@ -335,6 +262,8 @@ extern int fini(void)
 {
 	debug3("%s %s called", plugin_type, __func__);
 
+	slurm_curl_fini();
+
 	_free_tables();
 	xfree(datastr);
 	xfree(influxdb_conf.host);
@@ -356,6 +285,7 @@ extern void acct_gather_profile_p_conf_options(s_p_options_t **full_options,
 		{"ProfileInfluxDBDefault", S_P_STRING},
 		{"ProfileInfluxDBPass", S_P_STRING},
 		{"ProfileInfluxDBRTPolicy", S_P_STRING},
+		{"ProfileInfluxDBTimeout", S_P_UINT32},
 		{"ProfileInfluxDBUser", S_P_STRING},
 		{NULL} };
 
@@ -386,6 +316,9 @@ extern void acct_gather_profile_p_conf_set(s_p_hashtbl_t *tbl)
 			       "ProfileInfluxDBPass", tbl);
 		s_p_get_string(&influxdb_conf.rt_policy,
 			       "ProfileInfluxDBRTPolicy", tbl);
+		if (!s_p_get_uint32(&influxdb_conf.timeout,
+				    "ProfileInfluxDBTimeout", tbl))
+			influxdb_conf.timeout = DEFAULT_INFLUXDB_TIMEOUT;
 		s_p_get_string(&influxdb_conf.username,
 			       "ProfileInfluxDBUser", tbl);
 	}
@@ -591,47 +524,26 @@ extern int acct_gather_profile_p_add_sample_data(int table_id, void *data,
 	return SLURM_SUCCESS;
 }
 
-extern void acct_gather_profile_p_conf_values(List *data)
+extern void acct_gather_profile_p_conf_values(list_t **data)
 {
-	config_key_pair_t *key_pair;
+	add_key_pair(*data, "ProfileInfluxDBHost", "%s",
+		     influxdb_conf.host);
 
-	debug3("%s %s called", plugin_type, __func__);
+	add_key_pair(*data, "ProfileInfluxDBDatabase", "%s",
+		     influxdb_conf.database);
 
-	xassert(*data);
+	add_key_pair(*data, "ProfileInfluxDBDefault", "%s",
+		     acct_gather_profile_to_string(influxdb_conf.def));
 
-	key_pair = xmalloc(sizeof(config_key_pair_t));
-	key_pair->name = xstrdup("ProfileInfluxDBHost");
-	key_pair->value = xstrdup(influxdb_conf.host);
-	list_append(*data, key_pair);
+	/* skip over ProfileInfluxDBPass for security reasons */
 
-	key_pair = xmalloc(sizeof(config_key_pair_t));
-	key_pair->name = xstrdup("ProfileInfluxDBDatabase");
-	key_pair->value = xstrdup(influxdb_conf.database);
-	list_append(*data, key_pair);
+	add_key_pair(*data, "ProfileInfluxDBRTPolicy", "%s",
+		     influxdb_conf.rt_policy);
 
-	key_pair = xmalloc(sizeof(config_key_pair_t));
-	key_pair->name = xstrdup("ProfileInfluxDBDefault");
-	key_pair->value =
-		xstrdup(acct_gather_profile_to_string(influxdb_conf.def));
-	list_append(*data, key_pair);
-
-	key_pair = xmalloc(sizeof(config_key_pair_t));
-	key_pair->name = xstrdup("ProfileInfluxDBPass");
-	key_pair->value = xstrdup(influxdb_conf.password);
-	list_append(*data, key_pair);
-
-	key_pair = xmalloc(sizeof(config_key_pair_t));
-	key_pair->name = xstrdup("ProfileInfluxDBRTPolicy");
-	key_pair->value = xstrdup(influxdb_conf.rt_policy);
-	list_append(*data, key_pair);
-
-	key_pair = xmalloc(sizeof(config_key_pair_t));
-	key_pair->name = xstrdup("ProfileInfluxDBUser");
-	key_pair->value = xstrdup(influxdb_conf.username);
-	list_append(*data, key_pair);
-
-	return;
+	add_key_pair(*data, "ProfileInfluxDBTimeout", "%u",
+		     influxdb_conf.timeout);
 
+	/* skip over ProfileInfluxDBUser for security reasons */
 }
 
 extern bool acct_gather_profile_p_is_active(uint32_t type)
