@@ -50,7 +50,7 @@
 #include "pmixp_conn.h"
 #include "pmixp_dconn.h"
 
-#include "src/common/slurm_auth.h"
+#include "src/interfaces/auth.h"
 
 #define PMIXP_DEBUG_SERVER 1
 
@@ -366,13 +366,13 @@ pmixp_p2p_data_t _direct_proto = {
 
 static volatile int _was_initialized = 0;
 
-int pmixp_stepd_init(const stepd_step_rec_t *job, char ***env)
+int pmixp_stepd_init(const stepd_step_rec_t *step, char ***env)
 {
 	char *path;
 	int fd, rc;
 
-	if (SLURM_SUCCESS != (rc = pmixp_info_set(job, env))) {
-		PMIXP_ERROR("pmixp_info_set(job, env) failed");
+	if (SLURM_SUCCESS != (rc = pmixp_info_set(step, env))) {
+		PMIXP_ERROR("pmixp_info_set(step, env) failed");
 		goto err_info;
 	}
 
@@ -557,9 +557,9 @@ static int _auth_cred_verify(buf_t *buf, uid_t *uid)
  */
 
 static bool _serv_readable(eio_obj_t *obj);
-static int _serv_read(eio_obj_t *obj, List objs);
+static int _serv_read(eio_obj_t *obj, list_t *objs);
 static bool _serv_writable(eio_obj_t *obj);
-static int _serv_write(eio_obj_t *obj, List objs);
+static int _serv_write(eio_obj_t *obj, list_t *objs);
 static void _process_server_request(pmixp_base_hdr_t *hdr, buf_t *buf);
 
 
@@ -588,7 +588,7 @@ static bool _serv_readable(eio_obj_t *obj)
 	return true;
 }
 
-static int _serv_read(eio_obj_t *obj, List objs)
+static int _serv_read(eio_obj_t *obj, list_t *objs)
 {
 	/* sanity check */
 	xassert(NULL != obj );
@@ -648,7 +648,7 @@ static bool _serv_writable(eio_obj_t *obj)
 	return false;
 }
 
-static int _serv_write(eio_obj_t *obj, List objs)
+static int _serv_write(eio_obj_t *obj, list_t *objs)
 {
 	/* sanity check */
 	xassert(NULL != obj );
@@ -739,7 +739,7 @@ static int _process_extended_hdr(pmixp_base_hdr_t *hdr, buf_t *buf)
 		goto unlock;
 	}
 
-	switch (pmixp_dconn_progress_type(dconn)) {
+	switch (pmixp_dconn_progress_type()) {
 	case PMIXP_DCONN_PROGRESS_SW:{
 		/* this direct connection has fd that needs to be
 		 * polled to progress, use connection interface for that
@@ -769,7 +769,7 @@ static int _process_extended_hdr(pmixp_base_hdr_t *hdr, buf_t *buf)
 	}
 	default:
 		/* Should not happen */
-		xassert(0 && pmixp_dconn_progress_type(dconn));
+		xassert(0 && pmixp_dconn_progress_type());
 		/* TODO: handle this error */
 	}
 unlock:
@@ -833,7 +833,7 @@ static void _process_server_request(pmixp_base_hdr_t *hdr, buf_t *buf)
 				    hdr->seq, nodename, hdr->nodeid, coll->seq);
 			pmixp_debug_hang(0); /* enable hang to debug this! */
 			slurm_kill_job_step(pmixp_info_jobid(),
-					    pmixp_info_stepid(), SIGKILL);
+					    pmixp_info_stepid(), SIGKILL, 0);
 			xfree(nodename);
 			break;
 		} else if (PMIXP_COLL_REQ_SKIP == rc) {
@@ -1291,7 +1291,7 @@ _direct_conn_establish(pmixp_conn_t *conn, void *_hdr, void *msg)
 	if (!dconn) {
 		/* connection was refused because we already
 		 * have established connection
-		 * It seems that some sort of race condition occured
+		 * It seems that some sort of race condition occurred
 		 */
 		close(fd);
 		nodename = pmixp_info_job_host(hdr->nodeid);
