@@ -53,9 +53,6 @@
 /* State file name */
 #define SLINGSHOT_STATE_FILE "slingshot_state"
 
-/* New state file name (for atomic replacing) */
-#define SLINGSHOT_STATE_FILE_NEW "slingshot_state.new"
-
 /* Environment variable for libcxi library name (for dlopen()) */
 #define SLINGSHOT_CXI_LIB_VERSION_ENV "SLURM_SLINGSHOT_CXI_VERSION"
 
@@ -78,9 +75,9 @@
 #define SLINGSHOT_RDZV_GET_EN_FMT \
 	"/sys/class/cxi/cxi%d/device/properties/rdzv_get_en"
 
-/* File path to the default rdzv_get_en setting */
-#define SLINGSHOT_RDZV_GET_EN_DEFAULT_FILE \
-	"/sys/module/cxi_core/parameters/rdzv_get_en_default"
+/* File path format to the default rdzv_get_en setting */
+#define SLINGSHOT_RDZV_GET_EN_DEFAULT_FMT \
+	"/sys/module/cxi_%s/parameters/rdzv_get_en_default"
 
 extern int free_vnis; /* Number of free VNIs */
 
@@ -154,6 +151,7 @@ typedef struct slingshot_limits_set {
  * 'SwitchParameters' slurm.conf variable
  */
 typedef struct slingshot_config {
+	uint32_t destroy_retries; /* retry count for destroying services */
 	uint8_t single_node_vni;        /* Allocate VNIs for single-node apps */
 	uint8_t job_vni;                /* Allocate extra VNI per-job */
 	uint32_t tcs;                   /* Bitmap of default traffic classes */
@@ -300,7 +298,9 @@ extern void slingshot_release_collectives_job_step(slingshot_stepinfo_t *job);
 extern void slingshot_release_collectives_job(uint32_t job_id);
 /* config.c */
 extern void slingshot_free_config(void);
+extern bool slingshot_stepd_init(const char *switch_params);
 extern bool slingshot_setup_config(const char *switch_params);
+extern int slingshot_update_vni_table(void);
 extern bool slingshot_setup_job_vni_pool(job_record_t *job_ptr);
 extern bool slingshot_setup_job_step_vni(
 	slingshot_stepinfo_t *job, int node_cnt,
@@ -310,11 +310,6 @@ extern void slingshot_free_job_step_vni(slingshot_stepinfo_t *job);
 extern void slingshot_free_job_vni(uint32_t job_id);
 extern void slingshot_free_job_vni_pool(slingshot_jobinfo_t *job);
 extern void slingshot_free_jobinfo(slingshot_jobinfo_t *jobinfo);
-/* instant_on.c */
-extern bool slingshot_init_instant_on(void);
-extern void slingshot_fini_instant_on(void);
-extern bool slingshot_fetch_instant_on(slingshot_stepinfo_t *job,
-				       char *node_list, uint32_t node_cnt);
 /* setup_nic.c */
 extern bool slingshot_open_cxi_lib(slingshot_stepinfo_t *job);
 extern bool slingshot_create_services(slingshot_stepinfo_t *job, uint32_t uid,
