@@ -1,8 +1,7 @@
 /*****************************************************************************
  *  alloc.c - Slurm scrun job alloc handlers
  *****************************************************************************
- *  Copyright (C) 2023 SchedMD LLC.
- *  Written by Nathan Rini <nate@schedmd.com>
+ *  Copyright (C) SchedMD LLC.
  *
  *  This file is part of Slurm, a resource management program.
  *  For details, see <https://slurm.schedmd.com/>.
@@ -38,7 +37,6 @@
 
 #include "slurm/slurm.h"
 
-#include "src/common/conmgr.h"
 #include "src/common/cpu_frequency.h"
 #include "src/common/env.h"
 #include "src/common/net.h"
@@ -51,6 +49,8 @@
 #include "src/common/xmalloc.h"
 #include "src/common/xstring.h"
 
+#include "src/conmgr/conmgr.h"
+
 #include "src/scrun/scrun.h"
 
 /* max number of seconds to delay while waiting for job */
@@ -253,27 +253,25 @@ static void *_on_connection(conmgr_fd_t *con, void *arg)
 	return con;
 }
 
-static int _on_msg(conmgr_fd_t *con, slurm_msg_t *msg, void *arg)
+static int _on_msg(conmgr_fd_t *con, slurm_msg_t *msg, int unpack_rc, void *arg)
 {
 	int rc = SLURM_SUCCESS;
 	xassert(arg == con);
 
+	if (unpack_rc || !msg->auth_ids_set) {
+		error("%s: [%s] rejecting malformed RPC and closing connection: %s",
+		      __func__, conmgr_fd_get_name(con),
+		      slurm_strerror(unpack_rc));
+		slurm_free_msg(msg);
+		return unpack_rc;
+	}
+
 	switch (msg->msg_type)
 	{
 	case SRUN_PING:
 	{
 		/* if conmgr is alive then always respond success */
-		slurm_msg_t resp_msg;
-		return_code_msg_t rc_msg = {
-			.return_code = SLURM_SUCCESS,
-		};
-
-		response_init(&resp_msg, msg, RESPONSE_SLURM_RC, &rc_msg);
-		resp_msg.data_size = sizeof(rc_msg);
-
-		rc = conmgr_queue_write_msg(con, &resp_msg);
-		/* nothing to xfree() */
-
+		rc = slurm_send_rc_msg(msg, SLURM_SUCCESS);
 		debug("%s:[%s] srun RPC PING has been PONGED",
 		      __func__, conmgr_fd_get_name(con));
 		break;
@@ -339,12 +337,13 @@ static int _on_msg(conmgr_fd_t *con, slurm_msg_t *msg, void *arg)
 		      rpc_num2string(msg->msg_type));
 	}
 
+	slurm_free_msg(msg);
 	return rc;
 }
 
-static void _on_finish(void *arg)
+static void _on_finish(conmgr_fd_t *con, void *arg)
 {
-	conmgr_fd_t *con = arg;
+	xassert(con == arg);
 
 	if (get_log_level() > LOG_LEVEL_DEBUG) {
 		read_lock_state();
@@ -386,8 +385,8 @@ static uint32_t _setup_listener(void)
 	xassert(port > 0);
 	debug("%s: listening for srun RPCs on port=%hu", __func__, port);
 
-	if ((rc = conmgr_process_fd(CON_TYPE_RPC, fd, fd, events, NULL, 0,
-				    NULL)))
+	if ((rc = conmgr_process_fd(CON_TYPE_RPC, fd, fd, &events, CON_FLAG_NONE,
+				    NULL, 0, NULL)))
 		fatal("%s: conmgr refused fd=%d: %s",
 		      __func__, fd, slurm_strerror(rc));
 
@@ -400,9 +399,7 @@ static void _pending_callback(uint32_t job_id)
 }
 
 /* check allocation has all nodes ready */
-extern void check_allocation(conmgr_fd_t *con, conmgr_work_type_t type,
-			     conmgr_work_status_t status, const char *tag,
-			     void *arg)
+extern void check_allocation(conmgr_callback_args_t conmgr_args, void *arg)
 {
 	/* there must be only 1 thread that will call this at any one time */
 	static long delay = 1;
@@ -427,9 +424,9 @@ extern void check_allocation(conmgr_fd_t *con, conmgr_work_type_t type,
 		return;
 	}
 
-	if (status != CONMGR_WORK_STATUS_RUN) {
+	if (conmgr_args.status != CONMGR_WORK_STATUS_RUN) {
 		debug("%s: bailing due to callback status %s",
-		      __func__, conmgr_work_status_string(status));
+		      __func__, conmgr_work_status_string(conmgr_args.status));
 		stop_anchor(ESLURM_ALREADY_DONE);
 		return;
 	}
@@ -452,8 +449,7 @@ extern void check_allocation(conmgr_fd_t *con, conmgr_work_type_t type,
 			      __func__, state.jobid, delay);
 			unlock_state();
 		}
-		conmgr_add_delayed_work(NULL, check_allocation, delay, 0, NULL,
-					"check_allocation");
+		conmgr_add_work_delayed_fifo(check_allocation, NULL, delay, 0);
 	} else if ((rc == READY_JOB_FATAL) || !(rc & READY_JOB_STATE)) {
 		/* job failed! */
 		if (get_log_level() >= LOG_LEVEL_DEBUG) {
@@ -476,8 +472,7 @@ extern void check_allocation(conmgr_fd_t *con, conmgr_work_type_t type,
 			stop_anchor(rc);
 		} else {
 			/* we have a job now. see if creating is done */
-			conmgr_add_work(NULL, on_allocation,
-					 CONMGR_WORK_TYPE_FIFO, NULL, __func__);
+			conmgr_add_work_fifo(on_allocation, NULL);
 		}
 	}
 }
@@ -500,9 +495,10 @@ static void _alloc_job(void)
 		const char *val;
 		const env_vars_t *e = &env_vars[i];
 
-		if ((val = getenv(e->var)))
-			slurm_process_option_or_exit(&opt, e->type, val, true,
-						     false);
+		if ((val = getenv(e->var)) &&
+		    slurm_process_option(&opt, e->type, val, true, false))
+			fatal("%s: Unable to process environment variable %s=%s",
+			      __func__, e->var, val);
 	}
 
 	/* Process spank env options */
@@ -519,6 +515,11 @@ static void _alloc_job(void)
 	read_lock_state();
 	desc->name = xstrdup(state.id);
 	desc->container_id = xstrdup(state.id);
+	if (state.spank_job_env) {
+		desc->spank_job_env =
+			env_array_copy((const char **) state.spank_job_env);
+		desc->spank_job_env_size = envcount(state.spank_job_env);
+	}
 	unlock_state();
 	if (!desc->min_nodes || (desc->min_nodes == NO_VAL))
 		desc->min_nodes = 1;
@@ -592,9 +593,7 @@ static void _alloc_job(void)
 	slurm_free_resource_allocation_response_msg(alloc);
 }
 
-extern void get_allocation(conmgr_fd_t *con, conmgr_work_type_t type,
-			   conmgr_work_status_t status, const char *tag,
-			   void *arg)
+extern void get_allocation(conmgr_callback_args_t conmgr_args, void *arg)
 {
 	int rc;
 	job_info_msg_t *jobs = NULL;
@@ -678,10 +677,8 @@ extern void get_allocation(conmgr_fd_t *con, conmgr_work_type_t type,
 		if ((rc = _stage_in()))
 			stop_anchor(rc);
 		else
-			conmgr_add_work(NULL, on_allocation,
-					 CONMGR_WORK_TYPE_FIFO, NULL, __func__);
+			conmgr_add_work_fifo(on_allocation, NULL);
 	} else {
-		conmgr_add_delayed_work(NULL, check_allocation, 0, 1, NULL,
-					 "check_allocation");
+		conmgr_add_work_delayed_fifo(check_allocation, NULL, 0, 1);
 	}
 }
