@@ -50,20 +50,18 @@
 #include "src/slurmrestd/rest_auth.h"
 
 static pthread_rwlock_t paths_lock = PTHREAD_RWLOCK_INITIALIZER;
-static List paths = NULL;
+static list_t *paths = NULL;
 static data_parser_t **parsers; /* symlink to parser array */
 serializer_flags_t yaml_flags = SER_FLAGS_PRETTY;
 serializer_flags_t json_flags = SER_FLAGS_PRETTY;
 
-#define MAGIC 0xDFFEAAAE
 #define MAGIC_HEADER_ACCEPT 0xDF9EAABE
 
 typedef struct {
-	int magic;
+#define PATH_MAGIC 0xDFFEA1AE
+	int magic; /* PATH_MAGIC */
 	/* unique tag per path */
 	int tag;
-	/* handler's callback to call on match */
-	openapi_handler_t callback;
 	/* handler's ctxt callback to call on match */
 	const openapi_path_binding_t *op_path;
 	/* meta info from plugin */
@@ -87,11 +85,9 @@ static const char *_name(const on_http_request_args_t *args)
 
 static void _check_path_magic(const path_t *path)
 {
-	xassert(path->magic == MAGIC);
+	xassert(path->magic == PATH_MAGIC);
 	xassert(path->tag >= 0);
-	xassert(path->callback || path->op_path->callback);
-	xassert(!!path->callback !=
-		(path->op_path && !!path->op_path->callback));
+	xassert(path->op_path->callback);
 }
 
 static void _free_path(void *x)
@@ -103,7 +99,7 @@ static void _free_path(void *x)
 
 	_check_path_magic(path);
 
-	path->magic = ~MAGIC;
+	path->magic = ~PATH_MAGIC;
 	xfree(path);
 }
 
@@ -145,80 +141,6 @@ static int _match_path_key(void *x, void *ptr)
 		return 0;
 }
 
-static int _bind(const char *str_path, openapi_handler_t callback,
-		 const openapi_path_binding_t *op_path, int callback_tag,
-		 data_parser_t *parser, const openapi_resp_meta_t *meta)
-{
-	int path_tag;
-	path_t *path;
-
-	slurm_rwlock_wrlock(&paths_lock);
-
-	debug3("%s: binding %s to 0x%"PRIxPTR,
-	       __func__, str_path,
-	       (callback ? (uintptr_t) callback :
-		(uintptr_t) op_path->callback));
-
-	path_tag = register_path_tag(str_path);
-	if (path_tag == -1) {
-		debug("%s: skipping path: %s",
-		      __func__, str_path);
-		slurm_rwlock_unlock(&paths_lock);
-		return ESLURM_DATA_PATH_NOT_FOUND;
-	}
-
-	if ((path = list_find_first(paths, _match_path_key, &path_tag)))
-		goto exists;
-
-	/* add new path */
-	debug4("%s: new path %s with path_tag %d callback_tag %d",
-	       __func__, str_path, path_tag, callback_tag);
-	print_path_tag_methods(path_tag);
-
-	path = xmalloc(sizeof(*path));
-	path->magic = MAGIC;
-	path->tag = path_tag;
-	path->parser = parser;
-	list_append(paths, path);
-
-exists:
-	path->callback = callback;
-	path->op_path = op_path;
-	path->meta = meta;
-	path->callback_tag = callback_tag;
-
-	slurm_rwlock_unlock(&paths_lock);
-
-	return SLURM_SUCCESS;
-}
-
-extern int bind_operation_handler(const char *str_path,
-				  openapi_handler_t callback, int callback_tag)
-{
-	int rc;
-
-	if (!xstrstr(str_path, OPENAPI_DATA_PARSER_PARAM))
-		return _bind(str_path, callback, NULL, callback_tag, NULL,
-			     NULL);
-
-	for (int i = 0; parsers[i]; i++) {
-		char *path = xstrdup(str_path);
-
-		xstrsubstitute(path, OPENAPI_DATA_PARSER_PARAM,
-			       data_parser_get_plugin_version(parsers[i]));
-
-		rc = _bind(path, callback, NULL, callback_tag, parsers[i],
-			   NULL);
-
-		xfree(path);
-
-		if (rc)
-			return rc;
-	}
-
-	return SLURM_SUCCESS;
-}
-
 static int _add_binded_path(const char *path_str,
 			    const openapi_path_binding_t *op_path,
 			    const openapi_resp_meta_t *meta,
@@ -246,7 +168,7 @@ static int _add_binded_path(const char *path_str,
 	print_path_tag_methods(tag);
 
 	path = xmalloc(sizeof(*path));
-	path->magic = MAGIC;
+	path->magic = PATH_MAGIC;
 	path->tag = tag;
 	path->parser = parser;
 	path->op_path = op_path;
@@ -305,40 +227,6 @@ extern int bind_operation_path(const openapi_path_binding_t *op_path,
 	return rc;
 }
 
-static int _rm_path_callback(void *x, void *ptr)
-{
-	path_t *path = (path_t *)x;
-	bool mc = (path->callback == ptr);
-	bool mctxt = (path->op_path ? (path->op_path->callback == ptr) : false);
-
-	_check_path_magic(path);
-
-	if (!mc && !mctxt)
-		return 0;
-
-	debug5("%s: removing tag %d for callback=0x%"PRIxPTR,
-	       __func__, path->tag, (uintptr_t) ptr);
-	unregister_path_tag(path->tag);
-
-	return 1;
-}
-
-extern int unbind_operation_handler(openapi_handler_t callback)
-{
-	return ESLURM_NOT_SUPPORTED;
-}
-
-extern int unbind_operation_ctxt_handler(openapi_ctxt_handler_t callback)
-{
-	slurm_rwlock_wrlock(&paths_lock);
-
-	if (paths)
-		list_delete_all(paths, _rm_path_callback, callback);
-
-	slurm_rwlock_unlock(&paths_lock);
-	return SLURM_ERROR;
-}
-
 static int _operations_router_reject(const on_http_request_args_t *args,
 				     const char *err,
 				     http_status_code_t err_code,
@@ -429,7 +317,7 @@ static int _get_query(on_http_request_args_t *args, data_t **query,
 
 }
 
-static void _parse_http_accept_entry(char *entry, List l)
+static void _parse_http_accept_entry(char *entry, list_t *l)
 {
 	char *save_ptr = NULL;
 	char *token = NULL;
@@ -489,9 +377,9 @@ static void _http_accept_list_delete(void *x)
 	xfree(obj);
 }
 
-static List _parse_http_accept(const char *accept)
+static list_t *_parse_http_accept(const char *accept)
 {
-	List l = list_create(_http_accept_list_delete);
+	list_t *l = list_create(_http_accept_list_delete);
 	xassert(accept);
 	char *save_ptr = NULL;
 	char *token = NULL;
@@ -526,7 +414,7 @@ static int _resolve_mime(on_http_request_args_t *args, const char **read_mime,
 	}
 
 	if (args->accept) {
-		List accept = _parse_http_accept(args->accept);
+		list_t *accept = _parse_http_accept(args->accept);
 		http_header_accept_t *ptr = NULL;
 		list_itr_t *itr = list_iterator_create(accept);
 		while ((ptr = list_next(itr))) {
@@ -605,8 +493,7 @@ static int _resolve_mime(on_http_request_args_t *args, const char **read_mime,
 }
 
 static int _call_handler(on_http_request_args_t *args, data_t *params,
-			 data_t *query, openapi_handler_t callback,
-			 const openapi_path_binding_t *op_path,
+			 data_t *query, const openapi_path_binding_t *op_path,
 			 int callback_tag, const char *write_mime,
 			 data_parser_t *parser, const openapi_resp_meta_t *meta,
 			 const char *plugin)
@@ -616,25 +503,15 @@ static int _call_handler(on_http_request_args_t *args, data_t *params,
 	char *body = NULL;
 	http_status_code_t e;
 
-	if (callback) {
-		xassert(!op_path);
-		debug3("%s: [%s] BEGIN: calling handler: 0x%"PRIXPTR"[%d] for path: %s",
-		       __func__, _name(args), (uintptr_t) callback,
-		       callback_tag, args->path);
+	xassert(op_path);
+	debug3("%s: [%s] BEGIN: calling ctxt handler: 0x%"PRIXPTR"[%d] for path: %s",
+	       __func__, _name(args), (uintptr_t) op_path->callback,
+	       callback_tag, args->path);
 
-		rc = callback(_name(args), args->method, params, query,
-			      callback_tag, resp, args->context->auth, parser);
-	} else {
-		xassert(op_path);
-		debug3("%s: [%s] BEGIN: calling ctxt handler: 0x%"PRIXPTR"[%d] for path: %s",
-		       __func__, _name(args), (uintptr_t) op_path->callback,
-		       callback_tag, args->path);
-
-		rc = wrap_openapi_ctxt_callback(_name(args), args->method,
-						params, query, callback_tag,
-						resp, args->context->auth,
-						parser, op_path, meta);
-	}
+	rc = wrap_openapi_ctxt_callback(_name(args), args->method, params,
+					query, callback_tag, resp,
+					args->context->auth, parser, op_path,
+					meta);
 
 	/*
 	 * Clear auth context after callback is complete. Client has to provide
@@ -686,6 +563,23 @@ static int _call_handler(on_http_request_args_t *args, data_t *params,
 			e = HTTP_STATUS_CODE_ERROR_UNSUPPORTED_MEDIA_TYPE;
 		else if (rc == ESLURM_INVALID_JOB_ID)
 			e = HTTP_STATUS_CODE_ERROR_NOT_FOUND;
+		else if ((rc == SLURM_PROTOCOL_SOCKET_ZERO_BYTES_SENT) ||
+			 (rc == SLURM_COMMUNICATIONS_CONNECTION_ERROR) ||
+			 (rc == SLURM_COMMUNICATIONS_SEND_ERROR) ||
+			 (rc == SLURM_COMMUNICATIONS_RECEIVE_ERROR) ||
+			 (rc == SLURM_COMMUNICATIONS_SHUTDOWN_ERROR) ||
+			 (rc == SLURMCTLD_COMMUNICATIONS_CONNECTION_ERROR) ||
+			 (rc == SLURMCTLD_COMMUNICATIONS_SEND_ERROR) ||
+			 (rc == SLURMCTLD_COMMUNICATIONS_RECEIVE_ERROR) ||
+			 (rc == SLURMCTLD_COMMUNICATIONS_SHUTDOWN_ERROR) ||
+			 (rc == SLURMCTLD_COMMUNICATIONS_BACKOFF) ||
+			 (rc == ESLURM_DB_CONNECTION) ||
+			 (rc == ESLURM_PROTOCOL_INCOMPLETE_PACKET))
+			e = HTTP_STATUS_CODE_SRVERR_BAD_GATEWAY;
+		else if (rc == SLURM_PROTOCOL_SOCKET_IMPL_TIMEOUT)
+			e = HTTP_STATUS_CODE_SRVERR_GATEWAY_TIMEOUT;
+		else if (rc == SLURM_PROTOCOL_AUTHENTICATION_ERROR)
+			e = HTTP_STATUS_CODE_SRVERR_NETWORK_AUTH_REQ;
 
 		rc = _operations_router_reject(args, body, e, write_mime);
 	} else {
@@ -709,8 +603,8 @@ static int _call_handler(on_http_request_args_t *args, data_t *params,
 	}
 
 	debug3("%s: [%s] END: calling handler: (0x%"PRIXPTR") callback_tag %d for path: %s rc[%d]=%s status[%d]=%s",
-	       __func__, _name(args), (uintptr_t) callback, callback_tag,
-	       args->path, rc, slurm_strerror(rc), e,
+	       __func__, _name(args), (uintptr_t) op_path->callback,
+	       callback_tag, args->path, rc, slurm_strerror(rc), e,
 	       get_http_status_code_string(e));
 
 	xfree(body);
@@ -726,7 +620,6 @@ extern int operations_router(on_http_request_args_t *args)
 	data_t *params = NULL;
 	int path_tag;
 	path_t *path = NULL;
-	openapi_handler_t callback = NULL;
 	int callback_tag;
 	const char *read_mime = NULL, *write_mime = NULL, *plugin = NULL;
 	data_parser_t *parser = NULL;
@@ -759,14 +652,14 @@ extern int operations_router(on_http_request_args_t *args)
 	_check_path_magic(path);
 
 	/* clone over the callback info to release lock */
-	callback = path->callback;
 	callback_tag = path->callback_tag;
 	parser = path->parser;
 	slurm_rwlock_unlock(&paths_lock);
 
 	debug5("%s: [%s] found callback handler: (0x%"PRIXPTR") callback_tag=%d path=%s parser=%s",
-	       __func__, _name(args), (uintptr_t) callback, callback_tag,
-	       args->path, (parser ? data_parser_get_plugin(parser) : ""));
+	       __func__, _name(args), (uintptr_t) path->op_path->callback,
+	       callback_tag, args->path,
+	       (parser ? data_parser_get_plugin(parser) : ""));
 
 	if ((rc = _resolve_mime(args, &read_mime, &write_mime, &plugin)))
 		goto cleanup;
@@ -774,9 +667,8 @@ extern int operations_router(on_http_request_args_t *args)
 	if ((rc = _get_query(args, &query, read_mime)))
 		goto cleanup;
 
-	rc = _call_handler(args, params, query, callback, path->op_path,
-			   callback_tag, write_mime, parser, path->meta,
-			   plugin);
+	rc = _call_handler(args, params, query, path->op_path, callback_tag,
+			   write_mime, parser, path->meta, plugin);
 
 cleanup:
 	FREE_NULL_DATA(query);
