@@ -59,25 +59,25 @@
 #include <unistd.h>
 
 #include "src/common/fd.h"
-
 #include "src/common/hostlist.h"
 #include "src/common/log.h"
 #include "src/common/net.h"
 #include "src/common/proc_args.h"
 #include "src/common/read_config.h"
-#include "src/interfaces/auth.h"
-#include "src/interfaces/cli_filter.h"
-#include "src/interfaces/jobacct_gather.h"
 #include "src/common/slurm_opt.h"
 #include "src/common/slurm_protocol_api.h"
 #include "src/common/slurm_rlimits_info.h"
 #include "src/common/spank.h"
-#include "src/interfaces/switch.h"
 #include "src/common/uid.h"
 #include "src/common/xmalloc.h"
 #include "src/common/xsignal.h"
 #include "src/common/xstring.h"
 
+#include "src/interfaces/auth.h"
+#include "src/interfaces/cli_filter.h"
+#include "src/interfaces/jobacct_gather.h"
+#include "src/interfaces/switch.h"
+
 #include "src/bcast/file_bcast.h"
 
 #include "launch.h"
@@ -125,14 +125,13 @@ typedef struct _launch_app_data
  * forward declaration of static funcs
  */
 static void  _file_bcast(slurm_opt_t *opt_local, srun_job_t *job);
-static void  _launch_app(srun_job_t *job, List srun_job_list, bool got_alloc);
+static void _launch_app(srun_job_t *job, list_t *srun_job_list, bool got_alloc);
 static void *_launch_one_app(void *data);
 static void  _pty_restore(void);
 static void  _set_exit_code(void);
-static void  _set_node_alias(srun_job_t *job, List srun_job_list);
 static void  _setup_env_working_cluster(void);
-static void  _setup_job_env(srun_job_t *job, List srun_job_list,
-			    bool got_alloc);
+static void _setup_job_env(srun_job_t *job, list_t *srun_job_list,
+			   bool got_alloc);
 static void  _setup_one_job_env(slurm_opt_t *opt_local, srun_job_t *job,
 				bool got_alloc);
 static char *_uint16_array_to_str(int count, const uint16_t *array);
@@ -171,7 +170,7 @@ int srun(int ac, char **av)
 {
 	log_options_t logopt = LOG_OPTS_STDERR_ONLY;
 	bool got_alloc = false;
-	List srun_job_list = NULL;
+	list_t *srun_job_list = NULL;
 
 	slurm_init(NULL);
 	log_init(xbasename(av[0]), logopt, 0, NULL);
@@ -179,9 +178,7 @@ int srun(int ac, char **av)
 
 	if (cli_filter_init() != SLURM_SUCCESS)
 		fatal("failed to initialize cli_filter plugin");
-	if (slurm_cred_init() != SLURM_SUCCESS)
-		fatal("failed to initialize cred plugin");
-	if (switch_init(0) != SLURM_SUCCESS )
+	if (switch_g_init(false) != SLURM_SUCCESS )
 		fatal("failed to initialize switch plugins");
 
 	_setup_env_working_cluster();
@@ -190,9 +187,9 @@ int srun(int ac, char **av)
 	if (opt_list) {
 		if (!_enable_het_job_steps())
 			fatal("Job steps that span multiple components of a heterogeneous job are not currently supported");
-		create_srun_job((void **) &srun_job_list, &got_alloc, 0, 1);
+		create_srun_job((void **) &srun_job_list, &got_alloc);
 	} else
-		create_srun_job((void **) &job, &got_alloc, 0, 1);
+		create_srun_job((void **) &job, &got_alloc);
 
 	/*
 	 * Detect is process is in non-matching user namespace or UIDs
@@ -205,7 +202,6 @@ int srun(int ac, char **av)
 		debug3("%s: %ps GID %u and srun process GID %u mismatch",
 		       __func__, &job->step_id, job->gid, getgid());
 
-	_set_node_alias(job, srun_job_list);
 	_setup_job_env(job, srun_job_list, got_alloc);
 
 	/*
@@ -243,11 +239,9 @@ int srun(int ac, char **av)
 #ifdef MEMORY_LEAK_DEBUG
 	cli_filter_fini();
 	mpi_fini();
-	select_g_fini();
-	switch_fini();
+	switch_g_fini();
 	slurm_reset_all_options(&opt, false);
-	slurm_auth_fini();
-	slurm_conf_destroy();
+	slurm_fini();
 	log_fini();
 #endif /* MEMORY_LEAK_DEBUG */
 
@@ -278,7 +272,7 @@ static void *_launch_one_app(void *data)
 		launch_begin = true;
 		slurm_mutex_unlock(&launch_mutex);
 
-		pre_launch_srun_job(job, 0, 1, opt_local);
+		pre_launch_srun_job(job, opt_local);
 
 		slurm_mutex_lock(&launch_mutex);
 		launch_fini = true;
@@ -289,6 +283,14 @@ static void *_launch_one_app(void *data)
 	}
 	slurm_mutex_unlock(&launch_mutex);
 
+	/*
+	 * Update argv[0] after spank_local_user() so that S_JOB_ARGV holds the
+	 * original command line args in local context only.
+	 */
+	if (opt_local->srun_opt->bcast_flag) {
+		xfree(opt_local->argv[0]);
+		opt_local->argv[0] = xstrdup(opt_local->srun_opt->bcast_file);
+	}
 relaunch:
 	launch_common_set_stdio_fds(job, &cio_fds, opt_local);
 
@@ -319,7 +321,7 @@ relaunch:
 static void _reorder_het_job_recs(char **in_node_list, uint16_t **in_task_cnts,
 			       uint32_t ***in_tids, int total_nnodes)
 {
-	hostlist_t in_hl, out_hl;
+	hostlist_t *in_hl, *out_hl;
 	uint16_t *out_task_cnts = NULL;
 	uint32_t **out_tids = NULL;
 	char *hostname;
@@ -388,9 +390,9 @@ fini:	hostlist_destroy(in_hl);
 	xfree(out_tids);
 }
 
-static void _launch_app(srun_job_t *job, List srun_job_list, bool got_alloc)
+static void _launch_app(srun_job_t *job, list_t *srun_job_list, bool got_alloc)
 {
-	ListIterator opt_iter, job_iter;
+	list_itr_t *opt_iter, *job_iter;
 	slurm_opt_t *opt_local = NULL;
 	_launch_app_data_t *opts;
 	int total_ntasks = 0, total_nnodes = 0, step_cnt = 0, node_offset = 0;
@@ -407,7 +409,7 @@ static void _launch_app(srun_job_t *job, List srun_job_list, bool got_alloc)
 		first_job = (srun_job_t *) list_peek(srun_job_list);
 		if (!opt_list) {
 			if (first_job)
-				fini_srun(first_job, got_alloc, &global_rc, 0);
+				fini_srun(first_job, got_alloc, &global_rc);
 			fatal("%s: have srun_job_list, but no opt_list",
 			      __func__);
 		}
@@ -491,7 +493,7 @@ static void _launch_app(srun_job_t *job, List srun_job_list, bool got_alloc)
 				slurm_mutex_unlock(&step_mutex);
 				if (first_job) {
 					fini_srun(first_job, got_alloc,
-						  &global_rc, 0);
+						  &global_rc);
 				}
 				fatal("%s: job allocation count does not match request count (%d != %d)",
 				      __func__, list_count(srun_job_list),
@@ -534,8 +536,7 @@ static void _launch_app(srun_job_t *job, List srun_job_list, bool got_alloc)
 			opts->step_mutex  = &step_mutex;
 			srun_opt->het_step_cnt = het_job_step_cnt;
 
-			slurm_thread_create_detached(NULL, _launch_one_app,
-						     opts);
+			slurm_thread_create_detached(_launch_one_app, opts);
 		}
 		xfree(het_job_node_list);
 		xfree(het_job_task_cnts);
@@ -548,7 +549,7 @@ static void _launch_app(srun_job_t *job, List srun_job_list, bool got_alloc)
 		slurm_mutex_unlock(&step_mutex);
 
 		if (first_job)
-			fini_srun(first_job, got_alloc, &global_rc, 0);
+			fini_srun(first_job, got_alloc, &global_rc);
 	} else {
 		int i;
 		mpir_init(job->ntasks);
@@ -590,7 +591,7 @@ static void _launch_app(srun_job_t *job, List srun_job_list, bool got_alloc)
 		opts->opt_local   = &opt;
 		sropt.het_step_cnt = 1;
 		_launch_one_app(opts);
-		fini_srun(job, got_alloc, &global_rc, 0);
+		fini_srun(job, got_alloc, &global_rc);
 	}
 }
 
@@ -668,6 +669,7 @@ static void _setup_one_job_env(slurm_opt_t *opt_local, srun_job_t *job,
 	env->user_name = xstrdup(job->user_name);
 	env->gid = job->gid;
 	env->group_name = xstrdup(job->group_name);
+	env->oom_kill_step = opt_local->oom_kill_step;
 
 	if (srun_opt->pty) {
 		int fd = STDIN_FILENO;
@@ -707,9 +709,25 @@ static void _setup_one_job_env(slurm_opt_t *opt_local, srun_job_t *job,
 	}
 
 	setup_env(env, srun_opt->preserve_env);
+	/*
+	 * set_env_from_opts() could set job->env vars that are already set in
+	 * environ, but the values could be different (srun requests something
+	 * that is different from the job's environment) and we want the
+	 * job->env to take precedence.
+	 *
+	 * env_array_merge() overwrites anything in dest (the first argument)
+	 * with anything in source (the second argument). Thus, anything in
+	 * environ would overwrite anything already in job->env. The environ
+	 * vars could differ from vars in job->env, and at this point job->env
+	 * is the correct one.
+	 *
+	 * So, we need to first set the env vars from job->env to the environ,
+	 * then do the merge.
+	 */
 	set_env_from_opts(opt_local, &job->env,
 			  (job->het_job_offset == NO_VAL) ?
 			  -1 : job->het_job_offset);
+	env_array_set_environment(job->env);
 	env_array_merge(&job->env, (const char **)environ);
 
 	xfree(env->task_count);
@@ -717,16 +735,16 @@ static void _setup_one_job_env(slurm_opt_t *opt_local, srun_job_t *job,
 	xfree(env);
 }
 
-static void _setup_job_env(srun_job_t *job, List srun_job_list, bool got_alloc)
+static void _setup_job_env(srun_job_t *job, list_t *srun_job_list, bool got_alloc)
 {
-	ListIterator opt_iter, job_iter;
+	list_itr_t *opt_iter, *job_iter;
 	slurm_opt_t *opt_local;
 
 	if (srun_job_list) {
 		srun_job_t *first_job = list_peek(srun_job_list);
 		if (!opt_list) {
 			if (first_job)
-				fini_srun(first_job, got_alloc, &global_rc, 0);
+				fini_srun(first_job, got_alloc, &global_rc);
 			fatal("%s: have srun_job_list, but no opt_list",
 			      __func__);
 		}
@@ -737,7 +755,7 @@ static void _setup_job_env(srun_job_t *job, List srun_job_list, bool got_alloc)
 			if (!job) {
 				if (first_job) {
 					fini_srun(first_job, got_alloc,
-						  &global_rc, 0);
+						  &global_rc);
 				}
 				fatal("%s: job allocation count does not match request count (%d != %d)",
 				      __func__, list_count(srun_job_list),
@@ -758,7 +776,7 @@ static void _file_bcast(slurm_opt_t *opt_local, srun_job_t *job)
 {
 	srun_opt_t *srun_opt = opt_local->srun_opt;
 	struct bcast_parameters *params;
-	char *sep = NULL, *tmp = NULL;
+	char *tmp = NULL;
 	xassert(srun_opt);
 
 	if ((opt_local->argc == 0) || (opt_local->argv[0] == NULL))
@@ -768,33 +786,24 @@ static void _file_bcast(slurm_opt_t *opt_local, srun_job_t *job)
 	params->block_size = 8 * 1024 * 1024;
 	if (srun_opt->compress) {
 		params->compress = srun_opt->compress;
-	} else if ((tmp = xstrcasestr(slurm_conf.bcast_parameters,
-				      "Compression="))) {
-		tmp += 12;
-		sep = strchr(tmp, ',');
-		if (sep)
-			sep[0] = '\0';
+	} else if ((tmp = conf_get_opt_str(slurm_conf.bcast_parameters,
+					   "Compression="))) {
 		params->compress = parse_compress_type(tmp);
-		if (sep)
-			sep[0] = ',';
+		xfree(tmp);
 	}
 	params->exclude = xstrdup(srun_opt->bcast_exclude);
 	if (srun_opt->bcast_file && (srun_opt->bcast_file[0] == '/')) {
 		params->dst_fname = xstrdup(srun_opt->bcast_file);
-	} else if ((tmp = xstrcasestr(slurm_conf.bcast_parameters,
-				      "DestDir="))) {
-		/* skip past the key to the value */
-		params->dst_fname = xstrdup(tmp + 8);
-		/* handle a further comma-separated value */
-		if ((sep = xstrchr(params->dst_fname, ',')))
-			sep[0] = '\0';
+	} else if ((params->dst_fname =
+		    conf_get_opt_str(slurm_conf.bcast_parameters,
+				     "DestDir="))) {
 		xstrcatchar(params->dst_fname, '/');
 	} else {
 		xstrfmtcat(params->dst_fname, "%s/", opt_local->chdir);
 	}
 	if (srun_opt->send_libs)
 		params->flags |= BCAST_FLAG_SEND_LIBS;
-	params->fanout = 0;
+	params->tree_width = 0;
 	params->selected_step = xmalloc(sizeof(*params->selected_step));
 	params->selected_step->array_task_id = NO_VAL;
 	memcpy(&params->selected_step->step_id, &job->step_id,
@@ -814,8 +823,12 @@ static void _file_bcast(slurm_opt_t *opt_local, srun_job_t *job)
 		fatal("Failed to broadcast '%s'. Step launch aborted.",
 		      params->src_fname);
 
-	xfree(opt_local->argv[0]);
-	opt_local->argv[0] = xstrdup(params->dst_fname);
+	/*
+	 * Defer setting argv[0] to dst_fname till later point in
+	 * _launch_one_app(). Use bcast_file member as value placeholder.
+	 */
+	xfree(srun_opt->bcast_file);
+	srun_opt->bcast_file = xstrdup(params->dst_fname);
 
 	slurm_destroy_selected_step(params->selected_step);
 	xfree(params->dst_fname);
@@ -889,32 +902,6 @@ static void _set_exit_code(void)
 	}
 }
 
-static int _foreach_set_node_alias(void *x, void *arg)
-{
-	srun_job_t *job = x;
-	char *alias_list = NULL;
-
-	xassert(job);
-
-	if (job &&
-	    job->step_ctx &&
-	    job->step_ctx->step_resp &&
-	    job->step_ctx->step_resp->cred &&
-	    (alias_list = slurm_cred_get(job->step_ctx->step_resp->cred,
-					 CRED_DATA_JOB_ALIAS_LIST)))
-		set_nodes_alias(alias_list);
-
-	return SLURM_SUCCESS;
-}
-
-static void _set_node_alias(srun_job_t *job, List srun_job_list)
-{
-	if (srun_job_list)
-		list_for_each(srun_job_list, _foreach_set_node_alias, NULL);
-	else if (job)
-		_foreach_set_node_alias(job, NULL);
-}
-
 static void _pty_restore(void)
 {
 	/* STDIN is probably closed by now */
@@ -924,27 +911,43 @@ static void _pty_restore(void)
 
 static void _setup_env_working_cluster(void)
 {
-	char *working_env, *addr_ptr, *port_ptr, *rpc_ptr, *select_ptr;
+	char *working_env, *addr_ptr, *port_ptr, *rpc_ptr, *tmp = NULL;
 
 	if ((working_env = xstrdup(getenv("SLURM_WORKING_CLUSTER"))) == NULL)
 		return;
 
-	/* Format is cluster_name:address:port:rpc[:plugin_id_select] */
-	if (!(addr_ptr = strchr(working_env,  ':')) ||
-	    !(port_ptr = strchr(addr_ptr + 1, ':')) ||
-	    !(rpc_ptr  = strchr(port_ptr + 1, ':'))) {
-		error("malformed cluster addr and port in SLURM_WORKING_CLUSTER env var: '%s'",
-		      working_env);
-		exit(1);
+	/*
+	 * Format is cluster_name:[address]:port:rpc in 24.11+ or
+	 * cluster_name:address:port:rpc for older versions.  Disallow older
+	 * format two versions after 24.11.
+	 */
+	if (!(addr_ptr = strchr(working_env,  ':')))
+		goto error;
+	/* check for [] around the address */
+	if (addr_ptr[1] == '[') {
+		if (!(tmp = strchr(addr_ptr, ']')))
+			goto error;
+		port_ptr = strchr(tmp + 1, ':');
+	} else {
+		port_ptr = strchr(addr_ptr + 1, ':');
 	}
+	if (!port_ptr)
+		goto error;
+	if (!(rpc_ptr  = strchr(port_ptr + 1, ':')))
+		goto error;
 
+	if (tmp) {
+		/*
+		 * Delay increments add_ptr till now for new format to preserve
+		 * working_env in error message if failed ealier.
+		 */
+		*addr_ptr++ = '\0';
+		*tmp = '\0';
+	}
 	*addr_ptr++ = '\0';
 	*port_ptr++ = '\0';
 	*rpc_ptr++  = '\0';
 
-	if ((select_ptr = strchr(rpc_ptr, ':')))
-		*select_ptr++ = '\0';
-
 	if (xstrcmp(slurm_conf.cluster_name, working_env)) {
 		working_cluster_rec = xmalloc(sizeof(slurmdb_cluster_rec_t));
 		slurmdb_init_cluster_rec(working_cluster_rec, false);
@@ -956,11 +959,13 @@ static void _setup_env_working_cluster(void)
 		slurm_set_addr(&working_cluster_rec->control_addr,
 			       working_cluster_rec->control_port,
 			       working_cluster_rec->control_host);
-
-		if (select_ptr)
-			working_cluster_rec->plugin_id_select =
-				select_get_plugin_id_pos(strtol(select_ptr,
-								NULL, 10));
 	}
 	xfree(working_env);
+	unsetenv("SLURM_WORKING_CLUSTER");
+	return;
+
+error:
+	error("malformed cluster addr and port in SLURM_WORKING_CLUSTER env var: '%s'",
+	      working_env);
+	exit(1);
 }
