@@ -1,7 +1,8 @@
 /*****************************************************************************\
- *  test1.34.prog.c - Sleep for a second and terminate with SIGSEGV
+ *  prog1.32.prog.c - Simple signal catching test program for Slurm regression
+ *  test1.32. Report caught signals. Exit after SIGUSR1 and SIGUSR2 received.
  *****************************************************************************
- *  Copyright (C) 2008 Lawrence Livermore National Security.
+ *  Copyright (C) 2002 The Regents of the University of California.
  *  Produced at Lawrence Livermore National Laboratory (cf, DISCLAIMER).
  *  Written by Morris Jette <jette1@llnl.gov>
  *  CODE-OCEC-09-009. All rights reserved.
@@ -24,22 +25,58 @@
  *  with Slurm; if not, write to the Free Software Foundation, Inc.,
  *  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA.
 \*****************************************************************************/
+#include <signal.h>
+#include <stdio.h>
+#include <stdlib.h>
+#include <errno.h>
 #include <sys/types.h>
 #include <unistd.h>
-#include <stdio.h>
 
-int main (int argc, char *argv[])
+
+int sigusr1_cnt = 0, sigusr2_cnt = 0;
+
+void sig_handler(int sig)
 {
-	char *buffer;
-
-	/* Print the pid so the test1.34 script
-	 * can rm the core file core.getpid()
-	 * fflush() otherwise stdout loss in
-	 * the core dump.
-	 */
-	printf("%d\n", getpid());
+	switch (sig)
+	{
+		case SIGUSR1:
+			printf("Received SIGUSR1\n");
+			fflush(stdout);
+			sigusr1_cnt++;
+			break;
+		case SIGUSR2:
+			printf("Received SIGUSR2\n");
+			fflush(stdout);
+			sigusr2_cnt++;
+			break;
+		default:
+			printf("Received signal %d\n", sig);
+			fflush(stdout);
+	}
+}
+
+int main (int argc, char **argv)
+{
+	struct sigaction act;
+
+	act.sa_handler = sig_handler;
+	sigemptyset(&act.sa_mask);
+	act.sa_flags = 0;
+	if (sigaction(SIGUSR1, &act, NULL) < 0) {
+		perror("setting SIGUSR1 handler");
+		exit(2);
+	}
+	if (sigaction(SIGUSR2, &act, NULL) < 0) {
+		perror("setting SIGUSR2 handler");
+		exit(2);
+	}
+
+	printf("WAITING\n");
 	fflush(stdout);
-	buffer = (char *) 0;
-	buffer[1] = 'a';
-	return 0;
+
+	while ((sigusr1_cnt + sigusr2_cnt) < 2) {
+		sleep(1);
+	}
+	printf("FINI: sig1:%d sig2:%d\n", sigusr1_cnt, sigusr2_cnt);
+	exit(0);
 }
